/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.io.IOException;

import java.util.List;


/**
 * <p>
 * Defines an interface with the functionality to collect certain statistics of a given file. One implementation of
 * this interface collects certain statistical data from certain files against one or more metrics, such as the
 * length of a file, the number of lines in a file, etc. Each file processor has one unique alias, which is
 * configurable. A file processor also has an instance of file matcher which determines whether a file should be
 * processed by the file processor.
 * </p>
 *
 * <p>
 * In order to be used by <code>FileStatistics</code>, an implementation must provide a constructor with a
 * <code>Property</code> instance as argument where configuration properties are read from.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public interface FileProcessor {
    /**
     * <p>
     * Gets the list of supported metrics which statistical data are collected against. Each time, a new list should
     * be returned.
     * </p>
     *
     * @return the list of <code>Metric</code> instances representing supported metrics of this processor.
     */
    public List getSupportedMetrics();

    /**
     * <p>
     * Gets the alias of this processor. An alias uniquely identifying a processor in a <code>FileStatistics</code>
     * instance. In the final statistics report, the alias is also used to aggregate statistics of each processor.
     * The alias should always be non-empty string.
     * </p>
     *
     * @return the alias of this file processor
     */
    public String getAlias();

    /**
     * <p>
     * Gets the associated file matcher determining whether a file should be processed by this processor.
     * </p>
     *
     * @return the file matcher determining whether a file should be processed by this processor.
     */
    public FileMatcher getFileMatcher();

    /**
     * <p>
     * Processes the given file. The statistical data are collected and the statistical result is generated. All
     * statistics of supported metrics are collected and generated.
     * </p>
     *
     * @param file the file where statistical data are collected from.
     *
     * @return the statistical result collected from the given file.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     * @throws FileProcessingException if any other error occurs when collecting the statistical data.
     */
    public StatsResult process(File file) throws IOException, FileProcessingException;
}



