/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

import java.io.File;
import java.io.IOException;

import java.lang.reflect.InvocationTargetException;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines the main class of this component to provide the functionality to maintain a list of file processors and
 * collect statistics from a single file. This class also manages a <code>StatsReporting</code> instance used to
 * aggregate the statistical results and generate statistics reports. When processing a file, if the matching all
 * flag is <code>true</code>, any file processor, whose file matcher matches the file, collects the statistical
 * data. Otherwise, the first file processor in the list, whose file matcher matches the file, collects the
 * statistical data while remaining file processors are ignored. When using the default constructor, the list of
 * file processors is empty. <code>XmlStatsReporting</code> is used as the <code>StatsReporting</code> instance. The
 * instance of this class can also be created from a namespace in configuration.
 * </p>
 *
 * <p>
 * Property 'processors' must exists, which should contain at least one sub-property. Its sub-properties define the
 * list of file processors. Property 'classname' must exists under the sub-property defining the qualified class
 * name of the <code>FileProcessor</code> implementation. The sub-property itself is passed into the constructor of
 * the <code>FileProcessor</code> implementation. The order of the file processors is preserved as the oreder of
 * sub-properties in configuration. Property 'reporting' may exists whose sub-property 'classname' defining the
 * qualified class name of the <code>StatsReporting</code> implementation. The property 'reporting' itself is passed
 * into the constructor of the <code>StatsReporting</code> implementation. When the property is missing,
 * <code>XmlStatsReporting</code> is used. Property 'matchall' may exists. It defines a flag indicating whether all
 * matching file processors should be used or not. When the value is 'yes', 'on' or 'true' (case-insensitive), all
 * matching file processors should be used. Otherwise, only the first file processor should be used. When the
 * property is missing, only the first file processor should be used.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class FileStatistics {
    /** Represents the property name where the configurations of file processors are loaded. */
    public static final String PROCESSORS_PROPERTY_KEY = "processors";

    /**
     * Represents the property name where the configuration of <code>StatsReporting</code> implementation is loaded.
     */
    public static final String REPORTING_PROPERTY_KEY = "reporting";

    /** Represents the property name where the matching all flag is loaded. */
    public static final String MATCH_ALL_PROPERTY_KEY = "matchall";

    /**
     * Represents the property name where the qualified class names of file processors and/or statistics reporting
     * are loaded.
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /** Represents the property value of matching all processors flag representing <code>true</code>. */
    private static final String PROPERTY_VALUE_YES = "yes";

    /** Represents the property value of matching all processors flag representing <code>true</code>. */
    private static final String PROPERTY_VALUE_TRUE = "true";

    /** Represents the property value of matching all processors flag representing <code>true</code>. */
    private static final String PROPERTY_VALUE_ON = "on";

    /** Represents the mapping of aliases of file processors (keys) and the file processor instances (value). */
    private Map processors = new HashMap();

    /**
     * Represents the list of processors, since the order of the file processors is significant when looking for the
     * first file processor.
     */
    private List processorList = new ArrayList();

    /**
     * Represents the statistics reporting instance used to aggregate the statistical results and generate statistics
     * report.
     */
    private StatsReporting reporting = new XmlStatsReporting();

    /**
     * Represents the matching all flag. When it is <code>true</code>, all matching processors should be used.
     * Otherwise, only the first matching processor should be used.
     */
    private boolean matchAll = false;

    /**
     * <p>
     * Creates a new instance of <code>FileStatistics</code> class. The list of file processors is empty, and
     * <code>XmlStatsReporting</code> instance is used as the statistics reporting.
     * </p>
     */
    public FileStatistics() {
    }

    /**
     * <p>
     * Creates a new instance of <code>FileStatistics</code> class. The configurations are loaded from the given
     * namespace. The list of processors are loaded from the configuration. Statistics reporting instance and
     * matching all flag are also loaded from the configuration if available.
     * </p>
     *
     * @param namespace the namespace where the configurations are loaded.
     *
     * @throws NullPointerException if <code>namespace</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>namespace</code> is empty string.
     * @throws ConfigurationException if error occurs when loading configurations; or mandatory property is missing;
     *         or invalid property value exists.
     */
    public FileStatistics(String namespace) throws ConfigurationException {
        Validator.validateNotNullOrEmpty(namespace, "namespace");

        ConfigManager configManager = ConfigManager.getInstance();

        try {
            // Load processor property
            Property property = configManager.getPropertyObject(namespace, PROCESSORS_PROPERTY_KEY);

            if (property == null) {
                throw new ConfigurationException("Property " + PROCESSORS_PROPERTY_KEY + " cannot be missing.");
            }

            // Enumerate every sub-property
            Iterator subIter = property.list().iterator();

            while (subIter.hasNext()) {
                Property subProperty = (Property) subIter.next();

                // Extract the class name
                String className = subProperty.getValue(CLASSNAME_PROPERTY_KEY);

                // Create an instance and add the instance to the map
                addFileProcessor((FileProcessor) createInstance(className, subProperty, FileProcessor.class));
            }

            // Load reporting property
            property = configManager.getPropertyObject(namespace, REPORTING_PROPERTY_KEY);

            if (property != null) {
                // When the reporting property exists, instantiate the class
                String className = property.getValue(CLASSNAME_PROPERTY_KEY);

                // Set the reporting instance
                setReporting((StatsReporting) createInstance(className, property, StatsReporting.class));
            }

            // Load matching all processors flag
            String value = configManager.getString(namespace, MATCH_ALL_PROPERTY_KEY);

            matchAll = PROPERTY_VALUE_ON.equalsIgnoreCase(value) || PROPERTY_VALUE_TRUE.equalsIgnoreCase(value)
                || PROPERTY_VALUE_YES.equalsIgnoreCase(value);
        } catch (ConfigManagerException e) {
            throw new ConfigurationException("Loading configuration fails.", e);
        }
    }

    /**
     * <p>
     * Adds the file processor to the list of file processors. If a processor with the same alias already exists, the
     * existing processor is first removed. The new file processor is added to the end of the list.
     * </p>
     *
     * @param processor the file processor to be added
     *
     * @throws NullPointerException if <code>processor</code> is <code>null</code>.
     */
    public void addFileProcessor(FileProcessor processor) {
        Validator.validateNotNull(processor, "processor");

        Object obj = processors.put(processor.getAlias(), processor);

        if (obj != null) {
            // If the processor exists, it should be removed from the list as well.
            processorList.remove(obj);
        }

        processorList.add(processor);
    }

    /**
     * <p>
     * Removes the file processor with the given alias from the list of file processors. If such file processor does
     * not exist, <code>null</code> is returned.
     * </p>
     *
     * @param alias the alias of the file processor to be removed.
     *
     * @return the file processor being removed if the alias exists in the list; <code>null</code> otherwise.
     *
     * @throws NullPointerException if <code>alias</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    public FileProcessor removeFileProcessor(String alias) {
        Validator.validateNotNullOrEmpty(alias, "alias");

        FileProcessor processor = (FileProcessor) processors.remove(alias);

        if (processor != null) {
            // Remove the processor from list as well
            processorList.remove(processor);
        }

        return processor;
    }

    /**
     * <p>
     * Gets a file processor with the given alias from the list. If such file processor does not exist,
     * <code>null</code> is returned.
     * </p>
     *
     * @param alias the alias of the file processor to be retrieved.
     *
     * @return the file processor with the given alias if the alias exists in the list; <code>null</code> otherwise.
     *
     * @throws NullPointerException if <code>alias</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    public FileProcessor getFileProcessor(String alias) {
        Validator.validateNotNullOrEmpty(alias, "alias");

        return (FileProcessor) processors.get(alias);
    }

    /**
     * <p>
     * Gets the list of all file processors in this instance. A shallow copy of the list is returned.
     * </p>
     *
     * @return the list of all file processors in this instance.
     */
    public List getAllFileProcessors() {
        return new ArrayList(processorList);
    }

    /**
     * <p>
     * Clears all file processors in this instance.
     * </p>
     */
    public void clearAllFileProcessors() {
        processors.clear();
        processorList.clear();
    }

    /**
     * <p>
     * Gets the statistics reporting instance used to aggregate statistical results.
     * </p>
     *
     * @return the statistics reporting instance.
     */
    public StatsReporting getReporting() {
        return reporting;
    }

    /**
     * <p>
     * Sets the statistics reporting instance used to aggregate statistical results. The statistical results
     * aggregated in the previous reporting instance is discarded.
     * </p>
     *
     * @param reporting the stats reporting instance.
     *
     * @throws NullPointerException if <code>reporting</code> is <code>null</code>.
     */
    public void setReporting(StatsReporting reporting) {
        Validator.validateNotNull(reporting, "reporting");

        this.reporting = reporting;
    }

    /**
     * <p>
     * Gets a flag indicating whether all matching file processors whould be used or not.
     * </p>
     *
     * @return <code>true</code> if all matching file processors should be used; <code>false</code> otherwise.
     */
    public boolean isMatchAll() {
        return matchAll;
    }

    /**
     * <p>
     * Sets a flag indicating whether all matching file processors whould be used or not.
     * </p>
     *
     * @param matchAll a flag indicating whether all matching file processors whould be used or not.
     */
    public void setMatchAll(boolean matchAll) {
        this.matchAll = matchAll;
    }

    /**
     * <p>
     * Processes a single file. When the matching all flag is <code>true</code>, all file processors whose file
     * matchers successfully match the given file are used to collect the statistical data. Otherwise, the first
     * file processor in the list whose file matcher successfully matches the given file is used to collect the
     * statistical data only. The statistical data is aggregated by the statistics reporting instance. If no file
     * matcher can match the given file, no statistical data is collected. When error occurs during the process of
     * a file, the previously available statistical results before the error are accumulated.
     * </p>
     *
     * @param file the file whose statistical data is collected.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     * @throws FileStatisticsException if any other error occurs when collecting the statisitcal data.
     */
    public void processFile(File file) throws IOException, FileStatisticsException {
        Validator.validateNotNull(file, "file");

        // Enumerate every processor.
        Iterator iter = processorList.iterator();

        while (iter.hasNext()) {
            FileProcessor processor = (FileProcessor) iter.next();

            if (processor.getFileMatcher().match(file)) {
                // If the file matches, process it.
                reporting.aggregate(processor, file, processor.process(file));

                if (!matchAll) {
                    // If it is not required to match all processors, since one processor is found,
                    // no more processor should be considered.
                    break;
                }
            }
        }
    }

    /**
     * <p>
     * Creates a new instance of the class with the given qualified class name. The class should take a property in
     * configuration as the argument. The created instance is expected to be an instance of the given type.
     * </p>
     *
     * @param className the qualified class name whose instance is created.
     * @param property the property used as the argument of the constructor.
     * @param expectedType the expected type of the created instance.
     *
     * @return a new instance created from the given class using a property as the argument.
     *
     * @throws ConfigurationException if <code>className</code> is <code>null</code>; or the class cannot be found;
     *         or proper constructor cannot be found or accessed; or the given class cannot be instantiated; or the
     *         instance is not the given type.
     */
    private Object createInstance(String className, Property property, Class expectedType)
        throws ConfigurationException {
        if (className == null) {
            throw new ConfigurationException("Property classname cannot be missing in configuration.");
        }

        Class type;

        try {
            // Locate the class.
            type = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new ConfigurationException("The class cannot be found.", e);
        } catch (LinkageError e) {
            throw new ConfigurationException("Linkage fails.", e);
        } catch (SecurityException e) {
            throw new ConfigurationException("Access denied due to security error.", e);
        }

        if (!expectedType.isAssignableFrom(type)) {
            // If an instance of the class cannot be assigned to the expected type
            throw new ConfigurationException("The class " + className + " is not a sub-class of "
                + expectedType.getName() + ".");
        }

        try {
            // Locate the constructor and create an instance.
            return type.getConstructor(new Class[] {Property.class}).newInstance(new Object[] {property});
        } catch (NoSuchMethodException e) {
            throw new ConfigurationException("Proper constructor cannot be found.", e);
        } catch (InvocationTargetException e) {
            throw new ConfigurationException("Exception thrown by the constructor.", e);
        } catch (LinkageError e) {
            throw new ConfigurationException("Linkage fails.", e);
        } catch (IllegalAccessException e) {
            throw new ConfigurationException("Proper constructor cannot be accessed.", e);
        } catch (InstantiationException e) {
            throw new ConfigurationException("Instantiation fails.", e);
        } catch (SecurityException e) {
            throw new ConfigurationException("Access denied due to security error.", e);
        }
    }
}



