/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines a statistical result which is generated by the file processor and aggregated by the statistics report. The
 * statistical result is a map containing metrics as keys and integers as values. It provides functions to set and
 * retrieve statistics for each metric. It may also aggregate other statistical result to this result.
 * </p>
 *
 * <p>
 * This class is not thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public class StatsResult {
    /**
     * Represents the statistical result mapping. The keys are <code>Metric</code> instances representing the
     * measurement of the statistic. The values are statistical quantities (<code>Long</code> instances).
     */
    private Map results = new HashMap();

    /**
     * <p>
     * Creates a new instance of <code>StatsResult</code> class. The mapping is created.
     * </p>
     */
    public StatsResult() {
    }

    /**
     * <p>
     * Creates a new instance of <code>StatsResult</code> class. This is a copy constructor. The original statistical
     * result is given. The new instance is a shallow copy of the original statistical result.
     * </p>
     *
     * @param result the original statistical result to be copied.
     *
     * @throws NullPointerException if <code>result</code> is <code>null</code>.
     */
    public StatsResult(StatsResult result) {
        Validator.validateNotNull(result, "result");

        results.putAll(result.results);
    }

    /**
     * <p>
     * Sets a statistical quantity of the given metric. The previous quantity of the same metric is overwritten.
     * </p>
     *
     * @param metric the metric whose statistical value is set.
     * @param result the quantity of the given metric to be set.
     *
     * @throws NullPointerException if <code>metric</code> is <code>null</code>.
     */
    public void setStat(Metric metric, long result) {
        Validator.validateNotNull(metric, "metric");

        results.put(metric, new Long(result));
    }

    /**
     * <p>
     * Gets the quantity of the given metric. The metric should already exist in the statistical result.
     * </p>
     *
     * @param metric the metric whose statistical quantity is retrieved.
     *
     * @return the quantity of the given metric.
     *
     * @throws NullPointerException if <code>metric</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>metric</code> does not exist in the statistical result.
     */
    public long getStat(Metric metric) {
        Validator.validateNotNull(metric, "metric");

        Long result = (Long) results.get(metric);

        if (result == null) {
            throw new IllegalArgumentException("The metric does not exist in statistics.");
        }

        return result.longValue();
    }

    /**
     * <p>
     * Gets the list of all metrics in this statistical result. A shallow copy is returned.
     * </p>
     *
     * @return a list of all metrics in this statistical result.
     */
    public List getMetrics() {
        return new ArrayList(results.keySet());
    }

    /**
     * <p>
     * Aggregates the given result to this result. Every metric in the given result is processed. If the metric does
     * not exist in this result, it is added. Otherwise, the value is accumulated.
     * </p>
     *
     * @param result the result to be aggregated to this result.
     *
     * @throws NullPointerException if <code>result</code> is <code>null</code>.
     */
    public void aggregate(StatsResult result) {
        Validator.validateNotNull(result, "result");

        // Enumerate every result entry in the given result.
        Iterator iter = result.results.entrySet().iterator();

        while (iter.hasNext()) {
            Map.Entry entry = (Map.Entry) iter.next();
            Metric metric = (Metric) entry.getKey();
            Long stat = (Long) entry.getValue();

            if (results.containsKey(metric)) {
                // If the metric already exists in the result, the result should be accumulated.
                results.put(metric, new Long(((Long) results.get(metric)).longValue() + stat.longValue()));
            } else {
                // Otherwise, the result is added directly.
                results.put(metric, stat);
            }
        }
    }
}



