/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;

import com.topcoder.util.config.Property;

import java.lang.reflect.InvocationTargetException;


/**
 * <p>
 * Defines an abstract implementation of <code>FileProcessor</code> class. The common features of all file processors
 * such as aliases and file matchers are provided by this class. The alias and the file matcher can be provided
 * either programmatically or read from configuration.
 * </p>
 *
 * <p>
 * Property 'alias' must exists in configuration. Its value defines the unique alias of this processor. The alias
 * cannot be empty string. Property 'matcher' may exist in configuration. Such property is used as the argument when
 * creating the instance of file matcher. When property 'matcher' exists, sub-property 'matcher.classname' must
 * exists as well. Its value defines the qualified class name of <code>FileMatcher</code> implementation used by
 * this file processor. When property 'matcher' is missing, <code>AnyFileMatcher</code> is used.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileProcessorBase implements FileProcessor {
    /** Represents the property name where the alias of this processor is loaded. */
    public static final String ALIAS_PROPERTY_KEY = "alias";

    /** Represents the property name where the <code>FileMatcher</code> configuration is loaded. */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";

    /**
     * Represents the property name where the qualified class name of <code>FileMatcher</code> implementation is
     * used.
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /** Represents the unique alias of this file processor. */
    private String alias;

    /**
     * Represents the file matcher instance used to determine whether a file should be processed by this file
     * processor.
     */
    private FileMatcher matcher;

    /**
     * <p>
     * Creates a new instance of <code>FileProcessorBase</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    protected FileProcessorBase(String alias, FileMatcher matcher) {
        validateNotNullOrEmpty(alias, "alias");
        validateNotNull(matcher, "matcher");

        this.alias = alias;
        this.matcher = matcher;
    }

    /**
     * <p>
     * Creates a new instance of <code>FileProcessorBase</code> class. The alias and the file matcher are loaded from
     * configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if mandatory property is missing; or the file matcher instance cannot be
     *         created; or the alias is empty string.
     */
    protected FileProcessorBase(Property property) throws ConfigurationException {
        validateNotNull(property, "property");

        // Load alias
        alias = property.getValue(ALIAS_PROPERTY_KEY);
        validateNotNullOrEmptyConfiguration(alias, ALIAS_PROPERTY_KEY);

        // Load matcher
        Property matcherProperty = property.getProperty(FILE_MATCHER_PROPERTY_KEY);

        if (matcherProperty == null) {
            // If matcher does not exist in configuration, use AnyFileMatcher
            matcher = new AnyFileMatcher();
        } else {
            // Otherwise, instantiate the given class name
            String className = matcherProperty.getValue(CLASSNAME_PROPERTY_KEY);

            validateNotNullConfiguration(className, CLASSNAME_PROPERTY_KEY);

            try {
                matcher = (FileMatcher) Class.forName(className).getConstructor(new Class[] {Property.class})
                                             .newInstance(new Object[] {matcherProperty});
            } catch (ClassCastException e) {
                throw new ConfigurationException("The class is not a file matcher.", e);
            } catch (ClassNotFoundException e) {
                throw new ConfigurationException("The class cannot be found.", e);
            } catch (NoSuchMethodException e) {
                throw new ConfigurationException("Proper constructor cannot be found.", e);
            } catch (InvocationTargetException e) {
                throw new ConfigurationException("Exception thrown by the constructor.", e);
            } catch (LinkageError e) {
                throw new ConfigurationException("Linkage fails.", e);
            } catch (IllegalAccessException e) {
                throw new ConfigurationException("Proper constructor cannot be accessed.", e);
            } catch (InstantiationException e) {
                throw new ConfigurationException("Instantiation fails.", e);
            } catch (SecurityException e) {
                throw new ConfigurationException("Access denied due to security error.", e);
            }
        }
    }

    /**
     * <p>
     * Gets the alias of this file processor.
     * </p>
     *
     * @return the alias of this file processor.
     */
    public String getAlias() {
        return alias;
    }

    /**
     * <p>
     * Gets the file matcher instance used by this file processor.
     * </p>
     *
     * @return the file matcher instance used by this file processor.
     */
    public FileMatcher getFileMatcher() {
        return matcher;
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     */
    private void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new NullPointerException(name + " cannot be null.");
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> and empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    private void validateNotNullOrEmpty(String value, String name) {
        validateNotNull(value, name);

        if (value.trim().length() == 0) {
            throw new IllegalArgumentException(name + " cannot be empty string.");
        }
    }

    /**
     * <p>
     * Validates the value of a property in configuration. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the property.
     * @param name name of the property.
     *
     * @throws ConfigurationException if <code>value</code> is <code>null</code>.
     */
    private void validateNotNullConfiguration(Object value, String name)
        throws ConfigurationException {
        if (value == null) {
            throw new ConfigurationException("Property " + name + " cannot be missing in configuration.");
        }
    }

    /**
     * <p>
     * Validates the value of a property in configuration. The value cannot be <code>null</code> and empty string.
     * </p>
     *
     * @param value value of the property.
     * @param name name of the property.
     *
     * @throws ConfigurationException if <code>value</code> is <code>null</code> or empty string.
     */
    private void validateNotNullOrEmptyConfiguration(String value, String name)
        throws ConfigurationException {
        validateNotNullConfiguration(value, name);

        if (value.trim().length() == 0) {
            throw new ConfigurationException("Property " + name + " cannot be empty string.");
        }
    }
}



