/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Defines a concrete implementation of <code>FileProcessor</code> interface with the functionality to count number
 * of lines for files. The statistical result is stored using the metric named 'Line Count'. Lines are separated by
 * carriage return (known as CR), line feed (known as LF), or carriage return immediately following line feed (known
 * as CRLF).
 * </p>
 *
 * <p>
 * This class does not have additional configurations.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless and its super class is also thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 *
 * @see FileProcessorBase
 */
public class TextLineCounter extends FileProcessorBase {
    /** Represents the name of the metric supported by this processor. */
    public static final String NAME = "Line Count";

    /** Represents the display name of the metric supported by this processor. */
    public static final String DISPLAY_NAME = "line";

    /** Represents the description of this processor. */
    private static final String DESCRIPTION = "Count the number of lines for a text file.";

    /** Represents the metric describing the feature of this processor. */
    private static final Metric SUPPORTED_METRIC = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

    /**
     * <p>
     * Creates a new instance of <code>TextLineCounter</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    public TextLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * Creates a new instance of <code>TextLineCounter</code> class. The alias and the file matcher are loaded from
     * configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if mandatory property is missing; or the file matcher instance cannot be
     *         created; or the alias is empty string.
     */
    public TextLineCounter(Property property) throws ConfigurationException {
        super(property);
    }

    /**
     * <p>
     * Gets the list of supported metrics which statistical data are collected against. Each time, a new list should
     * be returned. For this class, the returned list contains only one <code>Metric</code> instance with the name
     * 'Line Count'.
     * </p>
     *
     * @return the list of <code>Metric</code> instances representing supported metrics of this processor.
     */
    public List getSupportedMetrics() {
        List metrics = new ArrayList();

        metrics.add(SUPPORTED_METRIC);

        return metrics;
    }

    /**
     * <p>
     * Processes the given file. The number of lines in the given file is counted.
     * </p>
     *
     * @param file the file where statistical data are collected from.
     *
     * @return the statistical result collected from the given file containing the number of lines in the file.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     */
    public StatsResult process(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file cannot be null.");
        }

        long lines = 0;
        BufferedReader input = null;

        try {
            // Create an buffered text reader using the default encoding
            input = new BufferedReader(new FileReader(file));

            while (input.readLine() != null) {
                // If a line can be read, increase the line number.
                ++lines;
            }
        } finally {
            if (input != null) {
                input.close();
            }
        }

        // Create the result
        StatsResult result = new StatsResult();

        // Set the line count
        result.setStat(SUPPORTED_METRIC, lines);

        return result;
    }
}



