/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import com.topcoder.util.config.ConfigManager;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * Demostrates the usage of this component. Normally, this component should be used via <code>Main</code>,
 * <code>ExtendedFileStatistics</code> or <code>FileStatistics</code>. It may be possible to use
 * <code>FileProcessor</code> and <code>FileMatcher</code> alone as well. Since this is a demostration, no assertion
 * is done in the test cases.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class DemoTestCase extends TestCase {
    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /** Represents the list of files to be deleted. */
    private List filesToDelete;

    /**
     * <p>
     * Sets up the test environment. The configurations are removed. The list of deleting files is created.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        // Remove all namespaces
        Iterator iter = configManager.getAllNamespaces();

        while (iter.hasNext()) {
            configManager.removeNamespace((String) iter.next());
        }

        filesToDelete = new ArrayList();
    }

    /**
     * <p>
     * Cleans up the test environment. The configurations are removed. The deleting files are deleted.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        // Remove all namespaces
        Iterator iter = configManager.getAllNamespaces();

        while (iter.hasNext()) {
            configManager.removeNamespace((String) iter.next());
        }

        // Delete files
        iter = filesToDelete.iterator();

        while (iter.hasNext()) {
            ((File) iter.next()).delete();
        }

        filesToDelete = null;
        configManager = null;
    }

    /**
     * <p>
     * Demostrates the usage of this component when the instances are created from the configuration. Processing
     * files/directories using <code>ExtendedFileStatistics</code> and generating reports are also demostrated here.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDemo1() throws Exception {
        File file = new File("test_files/report.tmp");

        filesToDelete.add(file);
        configManager.add("Main/Valid.xml");

        // Create a file statistics from configuration
        ExtendedFileStatistics fileStat = new ExtendedFileStatistics("com.topcoder.file.statistics.Main");

        // Set the base directory and clear all previous results
        fileStat.getReporting().reset(new File("test_files"));

        // Process a single file
        fileStat.processFile(new File("test_files/small.txt"));

        // Process a directory non-recursively
        fileStat.processDirectory(new File("test_files"));

        // Process a directory non-recursively
        fileStat.processDirectory(new File("test_files/Main"), false);

        // Process a directory recursively
        fileStat.processDirectory(new File("test_files/Deep"), true);

        // Print the report to the console
        fileStat.printReport();

        // Or print the report to a file
        fileStat.printReport(file);

        // Alternatively, get the reporting instance and generate the report
        String report = fileStat.getReporting().generateReport();
    }

    /**
     * <p>
     * Demostrates the usage of this component when the instances are created programmatically. Managing file
     * processors in <code>FileStatistics</code>, manipulating <code>FileProcessor</code> and
     * <code>FileMatcher</code> are also demostrated here.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDemo2() throws Exception {
        // Create a file statistics with no file processors
        FileStatistics fileStat = new FileStatistics();

        // Get the matching all flag
        boolean isMatchAll = fileStat.isMatchAll();

        // Get the statistics reporting instance
        StatsReporting reporting = fileStat.getReporting();

        // Set the matching all flag
        fileStat.setMatchAll(true);

        // Create a new statistics reporting instance
        reporting = new XmlStatsReporting(new File("test_files/transform.xsl"));

        // Set the statistics reporting instance
        fileStat.setReporting(reporting);

        // Create a new file matcher. Other file matcher can be created in a similar way
        FileMatcher matcher = new ExtensionMatcher("c");

        // Match a file
        boolean isMatch = matcher.match(new File("test_files/strange.c"));

        // Create a new file processor with alias and file matcher.
        // Other file processor can be created in a similar way
        FileProcessor processor = new CStyleLineCounter("C source line counter", matcher);

        // Get the alias of a file processor
        String alias = processor.getAlias();

        // Get the file matcher of a file processor
        matcher = processor.getFileMatcher();

        // Get the list of metrics produced by this file processor
        List metrics = processor.getSupportedMetrics();

        // Process a file and get the result
        StatsResult result = processor.process(new File("test_files/strange.c"));

        // Get a list of metrics in the result
        metrics = result.getMetrics();

        // Get result value of a metric
        long value = result.getStat((Metric) metrics.get(0));

        // Add the file processor
        fileStat.addFileProcessor(processor);

        // Get the file processor
        processor = fileStat.getFileProcessor("C source line counter");

        // Get the list of file processors
        List processors = fileStat.getAllFileProcessors();

        // Remove the file processor
        processor = fileStat.removeFileProcessor("C source line counter");

        // Clear all file processors
        fileStat.clearAllFileProcessors();
    }

    /**
     * <p>
     * Demostrates the usage of the command line tool in this component when all switches are specified.
     * </p>
     */
    public void testDemo3() {
        File file = new File("test_files/report.tmp");

        filesToDelete.add(file);

        // Specify all switches. The configuration is loaded from 'Main/Valid.xml' into namespace
        // 'test'. The processing is done recursively. The base directory is 'test_files'. The output
        // file is 'test_files/report.tmp' regardless of the base directory. The processing
        // files/directories are 'Deep' and 'small.txt' relative to base directory 'test_files'.
        Main.main(new String[] {
                "-c", "Main/Load.xml", "-n", "test", "-r", "-b", "test_files", "-o", "test_files/report.tmp", "Deep",
                "small.txt"
            });
    }

    /**
     * <p>
     * Demostrates the usage of the command line tool in this component when only files/directories are specified.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testDemo4() throws Exception {
        // The configuration must be preloaded
        configManager.add("Main/Valid.xml");

        // Specify no switch. The configuration should be preloaded into namespace
        // 'com.topcoder.file.statistics.Main'. The processing is not recursive. No base directory is
        // used. The report is printed to the console.
        Main.main(new String[] {"test_files/Deep", "test_files/small.txt"});
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(DemoTestCase.class);
    }
}

