/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.lang.reflect.Field;

import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;


/**
 * <p>
 * Tests functionality and error cases of <code>StatsResult</code> class. The accuracy test of <code>getStat</code>
 * is combined into the accuracy test of <code>setStat</code>, since setter and getter are usually tested as a pair.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class StatsResultTestCase extends TestCase {
    /** Represents the <code>StatsResult</code> instance used in tests. */
    private StatsResult stats;

    /** Represents the <code>Metric</code> instance used in tests. */
    private Metric metric;

    /**
     * <p>
     * Sets up the test environment. The test instances are created.
     * </p>
     */
    protected void setUp() {
        stats = new StatsResult();
        metric = new Metric("Name", "Display Name", "Description");
    }

    /**
     * <p>
     * Cleans up the test environment. The test instances are disposed.
     * </p>
     */
    protected void tearDown() {
        stats = null;
        metric = null;
    }

    /**
     * <p>
     * Tests constructor <code>StatsResult(StatsResult)</code> when the statistical result to be copied is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testStatsResultStatsResultNull() {
        try {
            new StatsResult(null);
            fail("The statistical result to be copied is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>setStats</code> when the given metric is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testSetStatsMetricNull() {
        try {
            stats.setStat(null, 0);
            fail("The metric is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>getStats</code> when the given metric is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testGetStatsMetricNull() {
        try {
            stats.getStat(null);
            fail("The metric is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>getStats</code> when the given metric does not exist in the statistical result. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testGetStatsMetricNotExist() {
        try {
            stats.getStat(metric);
            fail("The metric does not exist in the statistical result, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>aggregate</code> when the given statistical result is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testAggregateNull() {
        try {
            stats.aggregate(null);
            fail("The aggregating statistical result is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>StatsResult()</code>. The internal map should be empty.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testStatsResultAccuracy() throws Exception {
        stats = new StatsResult();

        Field field = StatsResult.class.getDeclaredField("results");

        try {
            field.setAccessible(true);
            assertTrue("The internal map should be empty.", ((Map) field.get(stats)).isEmpty());
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>StatsResult(StatsResult)</code>. The internal map of the given statistical
     * result should be copied to the new instance. When changing the content of the given statistical result, the
     * new instance should remain the unchanged.
     * </p>
     */
    public void testStatsResultStatsResultAccuracy() {
        // Set the content of the statistical result to be copied.
        stats.setStat(metric, 100);

        // Copy the statistical result.
        StatsResult newStats = new StatsResult(stats);

        // Verify the copied content
        assertEquals("The metric/value should be copied.", 100, newStats.getStat(metric));

        // Change the original statistical result
        stats.setStat(metric, 1);

        // Verify the new instance is indeed a copy.
        assertEquals("The metric/value should remain unchanged.", 100, newStats.getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>setStats</code> when the given metric does not exist in the result. The metric should
     * be added to the result together with the given value.
     * </p>
     */
    public void testSetStatsNotExistAccuracy() {
        // Add a new metric/value
        stats.setStat(metric, 100);

        // Verify
        assertEquals("The value should be correct.", 100, stats.getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>setStats</code> when the given metric already exists in the result. The old value of
     * the metric should be replaced by the new value.
     * </p>
     */
    public void testSetStatsExistAccuracy() {
        // Add a new metric/value
        stats.setStat(metric, 100);

        // Replace the metric/value
        stats.setStat(metric, 10);

        // Verify
        assertEquals("The value should be correct.", 10, stats.getStat(metric));
    }

    /**
     * <p>
     * Tests accuracy of <code>getMetrics</code>. The list of metrics should be returned correctly. However, the
     * order of the list items is not significant according to the component specification.
     * </p>
     */
    public void testGetMetricsAccuracy() {
        Metric metric2 = new Metric("another metric", "another metric", "another metric");

        // Set two metrics
        stats.setStat(metric, 100);
        stats.setStat(metric2, 200);

        // Expected metrics set
        Set expected = new HashSet();

        expected.add(metric);
        expected.add(metric2);

        // Get the metric list
        List list = stats.getMetrics();

        // Two metrics should be in the list.
        assertEquals("Two metrics should be in the list.", 2, list.size());

        // The items should be correct. The order is not important, so compare by set.
        assertEquals("The content of the list should be correct.", expected, new HashSet(list));
    }

    /**
     * <p>
     * Tests accuracy of <code>getMetrics</code>. A copy of the list should be returned. Each time, the returned list
     * should be different instances.
     * </p>
     */
    public void testGetMetricsCopyAccuracy() {
        // Set one metric/value
        stats.setStat(metric, 100);

        // Get the metric list
        List list = stats.getMetrics();

        // Each time, a new copy should be returned.
        assertNotSame("A new copy should be returned.", list, stats.getMetrics());
    }

    /**
     * <p>
     * Tests accuracy of <code>aggregate</code>. The given statistical result should be correctly aggregated. New
     * metric in the given statistical result should be added. Existing metric in the given statistical result
     * should be accumulated. Existing metric missing in the given statistical result should be preserved.
     * </p>
     */
    public void testAggregateAccuracy() {
        // Create the statistical result to be aggregated.
        StatsResult aggResult = new StatsResult();

        // Create two more metrics
        Metric newMetric = new Metric("new metric", "new metric", "new metric");
        Metric accuMetric = new Metric("accumulate metric", "accumulate metric", "accumulate metric");

        // Set metrics for the result to be aggregated.
        aggResult.setStat(newMetric, 10);
        aggResult.setStat(accuMetric, 20);

        // Set metrics already exists in the aggregated result
        stats.setStat(metric, 30);
        stats.setStat(accuMetric, 40);

        // Aggregate the result
        stats.aggregate(aggResult);

        // newMetric should be added to the aggregated result
        assertEquals("The new metric should be added.", 10, stats.getStat(newMetric));

        // metric should remain the same
        assertEquals("The existing metric should remain the same.", 30, stats.getStat(metric));

        // accuMetric should be accumulated.
        assertEquals("The metric existing in both results should be accumulated.", 20 + 40, stats.getStat(accuMetric));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(StatsResultTestCase.class);
    }
}



