/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.lang.reflect.Field;

import java.util.ArrayList;
import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>ExtensionMatcher</code> class. The <code>match(File)</code> method is
 * defined in its super class <code>FileNameMatcher</code>. The accuracy and error cases of this method is already
 * tested in the test case of that class. Therefore, here only the protected <code>match</code> method is tested.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class ExtensionMatcherTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>ExtensionMatcher</code> instance used in tests. */
    private ExtensionMatcher matcher;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        // The matcher instance contains four extensions
        List list = new ArrayList();

        list.add("bIn");
        list.add("TxT");
        list.add("Class");
        list.add("jAVA");
        matcher = new ExtensionMatcher(list);

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        matcher = null;
        configManager = null;
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(String)</code> when the given extension is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherStringNull() throws Exception {
        try {
            new ExtensionMatcher((String) null);
            fail("The extension is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(List)</code> when the given extension list is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     */
    public void testExtensionMatcherListNull() {
        try {
            new ExtensionMatcher((List) null);
            fail("The extension list is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(List)</code> when the given extension list is empty. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testExtensionMatcherListEmpty() {
        try {
            new ExtensionMatcher(new ArrayList());
            fail("The extension list is empty, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(List)</code> when the given extension list contains non-string
     * instance. Expect IllegalArgumentException.
     * </p>
     */
    public void testExtensionMatcherListNotString() {
        List list = new ArrayList();

        list.add(new Object());

        try {
            new ExtensionMatcher(list);
            fail("The extension list contains non-string instance, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(Property)</code> when the given property is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherPropertyNull() throws Exception {
        try {
            new ExtensionMatcher((Property) null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ExtensionMatcher(Property)</code> when the extension list is missing in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherPropertyExtensionMissing()
        throws Exception {
        configManager.add("ExtensionMatcher/Missing.xml");

        try {
            new ExtensionMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The extension list is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>ExtensionMatcher(String)</code>. The internal extension list should
     * contain only one extension, which is given as the argument.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherStringAccuracy() throws Exception {
        matcher = new ExtensionMatcher("jAvA");

        // Get the internal list
        Field field = ExtensionMatcher.class.getDeclaredField("extensions");
        List list;

        try {
            field.setAccessible(true);
            list = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify
        assertEquals("One extension should exist.", 1, list.size());
        assertEquals("The extension should be correct.", "jAvA", list.get(0));
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>ExtensionMatcher(List)</code>. The internal extension list should contain
     * all extensions, in a sorted order (case-insensitive), which are given as the argument. A copy of the given
     * list should be made.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherListAccuracy() throws Exception {
        List list = new ArrayList();

        list.add("bIn");
        list.add("TxT");
        list.add("Class");
        list.add("jAVA");

        matcher = new ExtensionMatcher(list);

        // Get the internal list
        Field field = ExtensionMatcher.class.getDeclaredField("extensions");
        List actual;

        try {
            field.setAccessible(true);
            actual = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify copy
        assertNotSame("A copy of the list should be made.", list, actual);

        // The list should be sorted.
        List expected = new ArrayList();

        expected.add("bIn");
        expected.add("Class");
        expected.add("jAVA");
        expected.add("TxT");

        // Verify the list
        assertEquals("The extension list should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>ExtensionMatcher(Property)</code>. The internal extension list should
     * contain all extensions, in a sorted order (case-insensitive), which are specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testExtensionMatcherPropertyAccuracy()
        throws Exception {
        configManager.add("ExtensionMatcher/Valid.xml");

        matcher = new ExtensionMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Get the internal list
        Field field = ExtensionMatcher.class.getDeclaredField("extensions");
        List actual;

        try {
            field.setAccessible(true);
            actual = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // The list should be sorted.
        List expected = new ArrayList();

        expected.add("BiN");
        expected.add("tXt");

        // Verify the list
        assertEquals("The extension list should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given extension exists in the extension list. The comparison
     * should be done case-insensitively.
     * </p>
     */
    public void testMatchStringExistAccuracy() {
        assertTrue("The file extension exists in the extension list.", matcher.match("whatsoever.TXT"));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given extension does not exist in the extension list. The
     * comparison should be done case-insensitively.
     * </p>
     */
    public void testMatchStringNotExistAccuracy() {
        assertFalse("The file extension does not exist in the extension list.", matcher.match("whatsoever.bak"));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when more than one dots exists in the file name. The extension should be
     * extracted correctly (from the last dot). The comparison should be done case-insensitively.
     * </p>
     */
    public void testMatchStringMultipleDotsAccuracy() {
        assertTrue("The file extension should be extracted correctly.", matcher.match("whatsoever.bak.java"));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(ExtensionMatcherTestCase.class);
    }
}



