/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;
import java.io.IOException;

import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>TextLineCounter</code> class. Since the constructors of this class
 * soly delegate the super class' constructors, the tests of constructors are not needed.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class TextLineCounterTestCase extends TestCase {
    /** Represents the <code>TextLineCounter</code> instance used in tests. */
    private FileProcessor processor;

    /**
     * <p>
     * Sets up the test environment. The test instance is created using <code>AnyFileMatcher</code>.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        processor = new TextLineCounter("alias", new AnyFileMatcher());
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        processor = null;
    }

    /**
     * <p>
     * Tests <code>process</code> when the given file is <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessNull() throws Exception {
        try {
            processor.process(null);
            fail("The file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>process</code> when I/O error occurs when processing the file. Expect IOException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessIOError() throws Exception {
        try {
            processor.process(new File("notexist"));
            fail("I/O error occurs when processing the file, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>getSupportedMetrics</code>. A list containing only one metric with the name 'Line
     * Count' and display name 'line' should be returned.
     * </p>
     */
    public void testGetSupportedMetricsAccuracy() {
        List list = processor.getSupportedMetrics();

        // Verify the size of list
        assertEquals("The metric list should contain only 1 metric.", 1, list.size());

        // Verify the content of metric
        Metric metric = (Metric) list.get(0);

        assertEquals("The name of the metric should be correct.", "Line Count", metric.getName());
        assertEquals("The display name of the metric should be correct.", "line", metric.getDisplayName());
        assertEquals("The description of the metric should be correct.",
            "Count the number of lines for a text file.", metric.getDescription());
    }

    /**
     * <p>
     * Tests accuracy of <code>getSupportedMetrics</code>. A copy of the list should be returned.
     * </p>
     */
    public void testGetSupportedMetricsCopyAccuracy() {
        List list = processor.getSupportedMetrics();

        assertNotSame("A copy of the list should be returned.", list, processor.getSupportedMetrics());
    }

    /**
     * <p>
     * Tests accuracy of <code>process</code>. The returned statistical result should contain only one metric. The
     * correct number of lines should be returned.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessAccuracy() throws Exception {
        Metric metric = (Metric) processor.getSupportedMetrics().get(0);
        StatsResult result = processor.process(new File("test_files/LineCount.txt"));

        // Verify metrics
        assertEquals("The statistical result should contain only one metric.", processor.getSupportedMetrics(),
            result.getMetrics());

        // Verify result
        assertEquals("The number of lines should be correct.", 20, result.getStat(metric));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(TextLineCounterTestCase.class);
    }
}



