/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.reporting;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReporting;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.File;

import java.lang.reflect.Field;


/**
 * <p>
 * Tests functionality and error cases of <code>XmlStatsReporting</code> class. Since the constructors of this class
 * soly delegate the super class' constructors, the tests of constructors are not needed.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class XmlStatsReportingTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>XmlStatsReporting</code> instance used in tests. */
    private StatsReporting reporting;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        reporting = new XmlStatsReporting();
        configManager = ConfigManager.getInstance();

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        configManager = null;
        reporting = null;
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(File)</code> when the given style sheet file is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     */
    public void testXmlStatsReportingFileNull() {
        try {
            new XmlStatsReporting((File) null);
            fail("The style sheet file is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(File)</code> when the given style sheet file does not exist. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testXmlStatsReportingFileNotExist() {
        try {
            new XmlStatsReporting(new File("notexist"));
            fail("The style sheet file does not exist, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(File)</code> when the given style sheet file is actually a
     * directory. Expect IllegalArgumentException.
     * </p>
     */
    public void testXmlStatsReportingFileIsDir() {
        try {
            new XmlStatsReporting(new File("test_files"));
            fail("The style sheet file is a directory, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(Property)</code> when the given property is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlStatsReportingPropertyNull() throws Exception {
        try {
            new XmlStatsReporting((Property) null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(Property)</code> when the style sheet file in configuration does
     * not exist. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlStatsReportingPropertyFileNotExist()
        throws Exception {
        configManager.add("XmlStatsReporting/NotExist.xml");

        try {
            new XmlStatsReporting(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The style sheet file in configuration does not exist, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>XmlStatsReporting(Property)</code> when the style sheet file in configuration is a
     * directory. Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlStatsReportingPropertyFileIsDir()
        throws Exception {
        configManager.add("XmlStatsReporting/IsDir.xml");

        try {
            new XmlStatsReporting(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The style sheet file in configuration is a directory, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>generateReport</code> when the style sheet file is invalid. Expect StatsReportingException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGenerateReportNotXSLT() throws Exception {
        reporting = new XmlStatsReporting(new File("test_files/TextReport.txt"));

        // Aggregate three results
        StatsResult result = new StatsResult();

        result.setStat(new Metric("metric1", "display_metric_1", "description1"), 100);
        reporting.aggregate(new TextLineCounter("processor1", new AnyFileMatcher()), new File("test_files/filea"),
            result);

        result = new StatsResult();
        result.setStat(new Metric("metric2", "display_metric_2", "description2"), 2);
        reporting.aggregate(new TextLineCounter("processor2", new AnyFileMatcher()), new File("conf/fileb"), result);

        result = new StatsResult();
        result.setStat(new Metric("metric2", "display_metric_2", "description2"), 3);
        reporting.aggregate(new TextLineCounter("processor2", new AnyFileMatcher()), new File("conf/filec"), result);

        try {
            reporting.generateReport();
            fail("The style sheet file is not an XSLT file, should throw StatsReportingException.");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>XmlStatsReporting(File)</code>. The style sheet file should be set correctly.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlStatsReportingFileAccuracy() throws Exception {
        Field field = XmlStatsReporting.class.getDeclaredField("stylesheet");

        reporting = new XmlStatsReporting(new File("test_files/transform.xsl"));

        // Verify the style sheet file
        try {
            field.setAccessible(true);
            assertEquals("The style sheet file should be set correctly.", new File("test_files", "transform.xsl"),
                field.get(reporting));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>XmlStatsReporting(Property)</code>. The style sheet file should be set as specified
     * in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testXmlStatsReportingPropertyAccuracy()
        throws Exception {
        Field field = XmlStatsReporting.class.getDeclaredField("stylesheet");

        configManager.add("XmlStatsReporting/Valid.xml");
        reporting = new XmlStatsReporting(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Verify the style sheet file
        try {
            field.setAccessible(true);
            assertEquals("The style sheet file should be set correctly.", new File("test_files", "transform.xsl"),
                field.get(reporting));
        } finally {
            field.setAccessible(false);
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>generateReport</code> without XSLT style sheet. The content of the report should
     * contain all statistical results including results for files, directories, processors and the overall result.
     * The report should be a formatted XML document.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGenerateReportNoXSLTAccuracy() throws Exception {
        // Aggregate three results
        StatsResult result = new StatsResult();

        result.setStat(new Metric("metric1", "display_metric_1", "description1"), 100);
        reporting.aggregate(new TextLineCounter("processor1", new AnyFileMatcher()), new File("test_files/filea"),
            result);

        result = new StatsResult();
        result.setStat(new Metric("metric2", "display_metric_2", "description2"), 2);
        reporting.aggregate(new TextLineCounter("processor2", new AnyFileMatcher()), new File("conf/fileb"), result);

        result = new StatsResult();
        result.setStat(new Metric("metric2", "display_metric_2", "description2"), 3);
        reporting.aggregate(new TextLineCounter("processor2", new AnyFileMatcher()), new File("conf/filec"), result);

        // Generate the report
        String report = reporting.generateReport();

        // Verify the report
        TestHelper.assertEquals("The generated XML report should be correct.", new File("test_files/XmlReport.xml"),
            report);
    }

    /**
     * <p>
     * Tests accuracy of <code>generateReport</code> using XSLT style sheet. The content of the report should
     * contain all statistical results including results for files, directories, processors and the overall result.
     * The report should be a transformed XML document using the given XSLT style sheet. The final result in this
     * test case should be an HTML file.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testGenerateReportXSLTAccuracy() throws Exception {
        reporting = new XmlStatsReporting(new File("test_files/transform.xsl"));

        // Aggregate three results
        StatsResult result = new StatsResult();

        result.setStat(new Metric("metric1", "display_metric_1", "description1"), 100);
        reporting.aggregate(new TextLineCounter("processor1", new AnyFileMatcher()), new File("test_files/filea"),
            result);

        result = new StatsResult();
        result.setStat(new Metric("metric2", "display_metric_2", "description2"), 2);
        reporting.aggregate(new TextLineCounter("processor2", new AnyFileMatcher()), new File("conf/fileb"), result);

        result = new StatsResult();
        result.setStat(new Metric("metric1", "display_metric_1", "description1"), 3);
        reporting.aggregate(new TextLineCounter("processor1", new AnyFileMatcher()), new File("conf/filec"), result);

        // Generate the report
        String report = reporting.generateReport();

        // Verify the report
        TestHelper.assertEquals("The generated XML report should be correct.",
            new File("test_files/XsltReport.htm"), report);
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(XmlStatsReportingTestCase.class);
    }
}



