/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.util.datavalidator.ObjectValidator;

/**
 * <p>
 * Defines a custom registered argument. The argument includes the switch name of the argument, which can be
 * specified in command line, a help message which can be printed to user, a require flag indicating whether
 * the switch is mandatory, and an optional validator which can be used to validate the value of the switch.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 2.0b
 */
public class ArgumentOption {
    /** Represents the switch name of this option. The switch name should be unique within a proposal. */
    private String switchName;

    /** Represents a help message of this option which should be printed to user. */
    private String helpMessage;

    /** Represents a flag indicating whether this option is required or not. */
    private boolean required;

    /** Represents an optional validator which can be used to validate the value of the switch. */
    private ObjectValidator validator = null;

    /**
     * <p>
     * Creates a new instance of <code>ArgumentOption</code> class. The switch name, help message and required flag are
     * given. Validation is not done to the value of the switch.
     * </p>
     *
     * @param switchName the switch name of this option.
     * @param helpMessage the help message of this option.
     * @param required the flag indicating whether the switch is required or not.
     * @throws NullPointerException if <code>switchName</code> or <code>helpMessage</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>switchName</code> or <code>helpMessage</code> is empty string.
     */
    public ArgumentOption(String switchName, String helpMessage, boolean required) {
        Validator.validateNotNullOrEmpty(switchName, "switchName");
        Validator.validateNotNullOrEmpty(helpMessage, "helpMessage");

        this.switchName = switchName;
        this.helpMessage = helpMessage;
        this.required = required;
    }

    /**
     * <p>
     * Creates a new instance of <code>ArgumentOption</code> class. The switch name, help message, required flag and the
     * switch value validator are given.
     * </p>
     *
     * @param switchName the switch name of this option.
     * @param helpMessage the help message of this option.
     * @param required the flag indicating whether the switch is required or not.
     * @param validator the validator used to validate the value of the switch.
     *
     * @throws NullPointerException if <code>switchName</code>, <code>helpMessage</code> or <code>validator</code> is
     *         <code>null</code>.
     * @throws IllegalArgumentException if <code>switchName</code> or <code>helpMessage</code> is empty string.
     */
    public  ArgumentOption(String switchName, String helpMessage, boolean required, ObjectValidator validator) {
        this(switchName, helpMessage, required);

        Validator.validateNotNull(validator, "validator");
        this.validator = validator;
    }

    /**
     * <p>
     * Gets the switch name of this option. The value cannot be <code>null</code> or empty string.
     * </p>
     *
     * @return the switch name of this option.
     */
    public String getSwitchName() {
        return switchName;
    }

    /**
     * <p>
     * Gets the help message of this option. The value cannot be <code>null</code> or empty string.
     * </p>
     *
     * @return the help message of this option.
     */
    public String getHelpMessage() {
        return helpMessage;
    }

    /**
     * <p>
     * Gets a flag indicating whether the switch is required or not.
     * </p>
     *
     * @return <code>true</code> if the switch is required; <code>false</code> otherwise.
     */
    public boolean isRequired() {
        return required;
    }

    /**
     * <p>
     * Gets the object validator used to validate the value of the switch. When the value is <code>null</code>,
     * no validation is done.
     * </p>
     *
     * @return the object validator used to validate the value of the switch when validation is done; <code>null</code>
     *         otherwise.
     */
    public ObjectValidator getValidator() {
        return validator;
    }
}
