/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.Property;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * <p>
 * Defines a concrete implementation of <code>FileMatcher</code> interface to provide the functionality to match a
 * file against a collection of file extensions. A file extension is a sub-string of the file name starting from the
 * last '.' character (excluding this character). When there is no '.' character, the file extension is empty
 * string. The matching is done case-insensitively.
 * </p>
 *
 * <p>
 * The collection of file extensions can be read from configuration under the property 'extensions'. This property
 * must contain at least one value. Empty strings are valid file extensions.
 * </p>
 *
 * <p>
 * This class is thead-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class ExtensionMatcher extends FileNameMatcher {
    /** Represents the property name in configuration to read the list of file extensions. */
    public static final String EXTENSIONS_PROPERTY_KEY = "extensions";

    /** Represents the string defining the separator between file name and file extension. */
    private static final String EXTENSION_SEPARATOR = ".";

    /** Represents an empty string. */
    private static final String EMPTY_STRING = "";

    /** Represents the set of file extensions in lower case to be matched. */
    private Set extensions = new HashSet();

    /**
     * <p>
     * Creates a new instance of <code>ExtensionMatcher</code> class. The list of file extensions contains only one
     * given extension.
     * </p>
     *
     * @param extension the file extension to be matched.
     *
     * @throws NullPointerException if <code>extension</code> is <code>null</code>.
     */
    public ExtensionMatcher(String extension) {
        Validator.validateNotNull(extension, "extension");

        extensions.add(extension.toLowerCase());
    }

    /**
     * <p>
     * Creates a new instance of <code>ExtensionMatcher</code> class. The list of file extensions is given. The list
     * should only contain strings, and cannot be empty list. Later modifications to the given list does not affect
     * the internal data.
     * </p>
     *
     * @param extensions the list of file extensions to be matched.
     *
     * @throws NullPointerException if <code>extensions</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>extensions</code> is empty; or contains instances other than
     *         strings.
     */
    public ExtensionMatcher(List extensions) {
        Validator.validateObjectCollection(extensions, "extensions", String.class);

        // Copy and convert the items in the list into set
        initializeExtensions(extensions);
    }

    /**
     * <p>
     * Creates a new instance of <code>ExtensionsMatcher</code> class. The list of file extensions is read from the
     * given property in configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if <code>property</code> does not contain the required sub-property; or the
     *         sub-property does not contain any value.
     */
    public ExtensionMatcher(Property property) throws ConfigurationException {
        Validator.validateNotNull(property, "property");

        String[] values = property.getValues(EXTENSIONS_PROPERTY_KEY);

        // Verify the property exists and is not empty
        Validator.validateNotNullOrEmptyConfiguration(values, EXTENSIONS_PROPERTY_KEY);

        // Copy and convert the items in the list into set
        initializeExtensions(Arrays.asList(values));
    }

    /**
     * <p>
     * Initializes the set of extensions using the given list. Every extension in the given list is first converted
     * into lower case since the comparison is required to be case-insensitive.
     * </p>
     *
     * @param extensions the list of extensions to be set.
     */
    private void initializeExtensions(List extensions) {
        Iterator iter = extensions.iterator();

        while (iter.hasNext()) {
            this.extensions.add(((String) iter.next()).toLowerCase());
        }
    }

    /**
     * <p>
     * Gets a flag indicating whether the extension of the given file name matches at least one of the file
     * extensions. The matching is done case-insensitively.
     * </p>
     *
     * @param filename the file name to be matched.
     *
     * @return <code>true</code> if the extension of the file name matches at least one of the file extensions;
     *         <code>false</code> otherwise.
     */
    protected boolean match(String filename) {
        int pos = filename.lastIndexOf(EXTENSION_SEPARATOR);
        String extension = (pos != -1) ? filename.substring(pos + 1) : EMPTY_STRING;

        // Compare the extension using lower case, as in the set.
        return extensions.contains(extension.toLowerCase());
    }
}

