/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileMatchingException;
import com.topcoder.file.type.FileType;
import com.topcoder.file.type.InvalidSignatureException;
import com.topcoder.file.type.InvalidXMLSignatureFileException;
import com.topcoder.file.type.MagicNumbers;

import com.topcoder.util.config.ConfigManagerException;

import java.io.File;
import java.io.IOException;


/**
 * <p>
 * Defines an abstract implementation of <code>FileMatcher</code> interface with the functionality to match a file
 * according to its content type. It uses Magic Numbers component to examine the content type of a file. Concrete
 * implementations should only override the abstract method.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is synchronized properly.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public abstract class FileTypeMatcher implements FileMatcher {
    /** Represents the magic numbers instance used to determine the file category. */
    private MagicNumbers magicNumbers = null;

    /** Represents a stub object used to synchronize <code>magicNumbers</code> properly. */
    private Object magicSync = new Object();

    /**
     * <p>
     * Creates a new instance of <code>FileTypeMatcher</code> class.
     * </p>
     */
    protected FileTypeMatcher() {
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file matches the criteria defined by the specific implementation. The
     * criteria is based on the content type of the file.
     * </p>
     *
     * @param file the file to be matched.
     *
     * @return <code>true</code> if the given file matches the criteria; <code>false</code> otherwise.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     * @throws FileMatchingException if any other error occurs when matching the file.
     */
    public boolean match(File file) throws IOException, FileMatchingException {
        Validator.validateNotNull(file, "file");

        FileType type;

        synchronized (magicSync) {
            if (magicNumbers == null) {
                try {
                    // Create an instance of magic number component.
                    magicNumbers = new MagicNumbers();
                } catch (InvalidSignatureException e) {
                    throw new FileMatchingException("The magic number signature has invalid format.", e);
                } catch (ConfigManagerException e) {
                    throw new FileMatchingException("Loading configuration property for magic number fails.", e);
                } catch (InvalidXMLSignatureFileException e) {
                    throw new FileMatchingException("The magic number configuration is invalid.", e);
                }
            }

            // Check the file type. It should also be synchronized,
            // since the magic number component is not thread-safe.
            type = magicNumbers.determineFileType(file);
        }

        return (type == null) ? false : match(type);
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file content type matches the criteria defined by the specific
     * implementation.
     * </p>
     *
     * @param type the file content type to be matched.
     *
     * @return <code>true</code> if the file content type matches the criteria; <code>false</code> otherwise.
     *
     * @throws FileMatchingException if any error occurs when matching the file content type.
     */
    protected abstract boolean match(FileType type) throws FileMatchingException;
}

