/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.Property;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;


/**
 * <p>
 * Defines a concrete implementation of <code>FileMatcher</code> interface to provide the functionality to match a
 * file against a list of regular expressions.
 * </p>
 *
 * <p>
 * The list of regular expressions can be read from configuration under the property 'patterns'. This property must
 * contain at least one value. All values should be valid regular expressions. Empty strings are not allowed.
 * </p>
 *
 * <p>
 * This class is thead-safe, since it is immutable.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class RegexMatcher extends FileNameMatcher {
    /** Represents the property name in configuration to read the list of regular expressions. */
    public static final String PATTERNS_PROPERTY_KEY = "patterns";

    /** Represents the list of <code>Pattern</code> instances defining the regular expressions. */
    private List patterns = new ArrayList();

    /**
     * <p>
     * Creates a new instance of <code>RegexMatcher</code> class. The list of regular expressions contains only one
     * given pattern.
     * </p>
     *
     * @param pattern the regular expression to be matched.
     *
     * @throws NullPointerException if <code>pattern</code> is <code>null</code>.
     */
    public RegexMatcher(Pattern pattern) {
        Validator.validateNotNull(pattern, "pattern");

        patterns.add(pattern);
    }

    /**
     * <p>
     * Creates a new instance of <code>RegexMatcher</code> class. The list of regular expressions is given. The list
     * should only contain <code>Pattern</code> instances, and cannot be empty list. Later modifications to the
     * given list does not affect the internal data.
     * </p>
     *
     * @param patterns the list of regular expressions to be matched.
     *
     * @throws NullPointerException if <code>patterns</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>patterns</code> is empty; or contains instances other than
     *         <code>Pattern</code>.
     */
    public RegexMatcher(List patterns) {
        Validator.validateObjectCollection(patterns, "patterns", Pattern.class);

        this.patterns.addAll(patterns);
    }

    /**
     * <p>
     * Creates a new instance of <code>RegexMatcher</code> class. The list of regular expressions is read from the
     * given property in configuration.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if <code>property</code> does not contain the required sub-property; or the
     *         sub-property does not contain any value; or the values contain invalid regular expressions.
     */
    public RegexMatcher(Property property) throws ConfigurationException {
        Validator.validateNotNull(property, "property");

        String[] values = property.getValues(PATTERNS_PROPERTY_KEY);

        // Verify the property values
        Validator.validateStringArrayConfiguration(values, PATTERNS_PROPERTY_KEY);

        // Add regular patterns
        for (int i = 0; i < values.length; ++i) {
            try {
                // Compile and add the pattern
                patterns.add(Pattern.compile(values[i]));
            } catch (PatternSyntaxException e) {
                throw new ConfigurationException("Property " + PATTERNS_PROPERTY_KEY + " contains invalid pattern.", e);
            }
        }
    }

    /**
     * <p>
     * Gets a flag indicating whether the given file name matches at least one of the regular expressions.
     * </p>
     *
     * @param filename the file name to be matched.
     *
     * @return <code>true</code> if the file name matches at least one of the regular expressions; <code>false</code>
     *         otherwise.
     */
    protected boolean match(String filename) {
        // Try to match every pattern in the list.
        Iterator iter = patterns.iterator();

        while (iter.hasNext()) {
            Pattern pattern = (Pattern) iter.next();

            // If one pattern can be found in the filename, the filename matches.
            if (pattern.matcher(filename).find()) {
                return true;
            }
        }

        // If no pattern matches, it is not a match.
        return false;
    }
}




