/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import java.util.Collection;
import java.util.Iterator;


/**
 * <p>
 * Defines utilities to validate variables.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
final class Validator {
    /**
     * <p>
     * Creates a new instance of <code>Validator</code> class. This private constructor prevents the creation of a
     * new instance.
     * </p>
     */
    private Validator() {
    }

    /**
     * <p>
     * Validates the value of a property in configuration. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the property.
     * @param name name of the property.
     *
     * @throws ConfigurationException if <code>value</code> is <code>null</code>.
     */
    public static void validateNotNullConfiguration(Object value, String name)
        throws ConfigurationException {
        if (value == null) {
            throw new ConfigurationException("Property " + name + " cannot be missing in configuration.");
        }
    }

    /**
     * <p>
     * Validates the array of values of a property. The array of values cannot be <code>null</code> and empty
     * string.
     * </p>
     *
     * @param value the array of values of the property.
     * @param name name of the property.
     *
     * @throws ConfigurationException if <code>value</code> is <code>null</code> or empty array.
     */
    public static void validateNotNullOrEmptyConfiguration(Object[] value, String name)
        throws ConfigurationException {
        validateNotNullConfiguration(value, name);

        if (value.length == 0) {
            throw new ConfigurationException("Property " + name + " must contain at least one value.");
        }
    }

    /**
     * <p>
     * Validates the value of a variable with type <code>Object</code>. The value cannot be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     */
    public static void validateNotNull(Object value, String name) {
        if (value == null) {
            throw new NullPointerException(name + " cannot be null.");
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be empty string, but may be <code>null</code>.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    public static void validateNotEmpty(String value, String name) {
        if (value != null) {
            if (value.trim().length() == 0) {
                throw new IllegalArgumentException(name + " cannot be empty string.");
            }
        }
    }

    /**
     * <p>
     * Validates the values of a property in configuration. The value array cannot be <code>null</code>. It also
     * cannot contain empty strings.
     * </p>
     *
     * @param value value of the property.
     * @param name name of the property.
     *
     * @throws ConfigurationException if <code>value</code> is <code>null</code> or empty; or contains empty string.
     */
    public static void validateStringArrayConfiguration(String[] value, String name)
        throws ConfigurationException {
        // Verify the property exists and is not empty
        Validator.validateNotNullOrEmptyConfiguration(value, name);

        // Verify each element is non-empty string
        for (int i = 0; i < value.length; ++i) {
            if (value[i].trim().length() == 0) {
                throw new ConfigurationException("Property " + name + " cannot contain empty strings.");
            }
        }
    }

    /**
     * <p>
     * Validates the value of a string variable. The value cannot be <code>null</code> and empty string.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>value</code> is empty string.
     */
    public static void validateNotNullOrEmpty(String value, String name) {
        validateNotNull(value, name);
        validateNotEmpty(value, name);
    }

    /**
     * <p>
     * Validates the value of a string collection. The value cannot be <code>null</code> or empty collection. The
     * collection should only contain instances of the given type.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     * @param type type of the elements in the collection.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>value</code> is empty collection; or <code>value</code> contains
     *         <code>null</code> or instances other than the given type.
     */
    public static void validateObjectCollection(Collection value, String name, Class type) {
        validateNotNull(value, name);

        if (value.isEmpty()) {
            // If empty, not legal.
            throw new IllegalArgumentException(name + " cannot be empty collection.");
        }

        // Enumerate every element in collection.
        Iterator iter = value.iterator();

        while (iter.hasNext()) {
            Object obj = iter.next();

            if (!type.isInstance(obj)) {
                // If null or illegal instance exists.
                throw new IllegalArgumentException(name + " cannot contain instances other than " + type.getName()
                    + ".");
            }
        }
    }

    /**
     * <p>
     * Validates the value of a string collection. The value cannot be <code>null</code> or empty collection. The
     * collection should only contain non-empty string instances.
     * </p>
     *
     * @param value value of the variable.
     * @param name name of the variable.
     *
     * @throws NullPointerException if <code>value</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>value</code> is empty collection; or <code>value</code> contains
     *         <code>null</code>, instances other than strings or empty strings.
     */
    public static void validateStringCollection(Collection value, String name) {
        validateNotNull(value, name);

        if (value.isEmpty()) {
            // If empty, not legal.
            throw new IllegalArgumentException(name + " cannot be empty collection.");
        }

        // Enumerate every element in collection.
        Iterator iter = value.iterator();

        while (iter.hasNext()) {
            Object obj = iter.next();

            if (!(obj instanceof String)) {
                // If null or non-string instance exists.
                throw new IllegalArgumentException(name + " cannot contain instances other than strings.");
            }

            if (((String) obj).trim().length() == 0) {
                // If empty string exists.
                throw new IllegalArgumentException(name + " cannot contain empty strings.");
            }
        }
    }
}




