/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.processor.linecounter;

import com.topcoder.file.statistics.ArgumentHandler;
import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;

import com.topcoder.util.config.Property;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;


/**
 * <p>
 * Defines a concrete implementation of <code>FileProcessor</code> interface with the functionality to count number
 * of C-style source code lines for files. The statistical result is stored using the metric named 'Line Count'.
 * Whenever ';' or '{' is encountered outside the string literals and comments, the following contexts are
 * considered in a new line. This class can be useful when counting lines of C/C++/Java/C# source codes. However,
 * when C# '&#64;' string literals are used in the source code, the line counting may be incorrect.
 * </p>
 *
 * <p>
 * Property 'support_at_literal' can exist under the given property. When the value is 'on', 'yes' or 'true'
 * (case-insensitive), the '@'-literals are supported. Otherwise, the '@'-literals are not supported. By default, it is
 * not supported. When specifying from command line, if the switch 'support_at_literal' exists, the '@'-literals are
 * supported. Otherwise, the '@'-literals are not supported.
 * </p>
 *
 * <p>
 * This class is thread-safe, since it is stateless and its super class is also thread-safe.
 * </p>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 *
 * @see FileProcessorBase
 */
public class CStyleLineCounter extends FileProcessorBase implements ArgumentHandler {
    /** Represents the name of the metric supported by this processor. */
    public static final String NAME = "Line Count";

    /** Represents the display name of the metric supported by this processor. */
    public static final String DISPLAY_NAME = "line";

    /**
     * Represents the property name where the flag indicating whether '@'-literal should be supported when counting the
     * lines.
     *
     * @since 2.0b
     */
    public static final String SUPPORT_AT_LITERAL_PROPERTY_KEY = "support_at_literal";

    /** Represents the description of this processor. */
    private static final String DESCRIPTION = "Count the number of lines for a C-style source file.";

    /** Represents the metric describing the feature of this processor. */
    private static final Metric SUPPORTED_METRIC = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_YES = "yes";

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_TRUE = "true";

    /**
     * Represents the property value of matching all processors flag representing <code>true</code>.
     *
     * @since 2.0b
     */
    private static final String PROPERTY_VALUE_ON = "on";

    /**
     * Represents the argument option specifying whether '@'-literals should be supported or not.
     *
     * @since 2.0b
     */
    private static final ArgumentOption SUPPORT_AT_LITERAL_OPTION = new ArgumentOption(
        SUPPORT_AT_LITERAL_PROPERTY_KEY, "Whether '@'-literals in C# should be supported or not.", false);

    /**
     * Represents the flag indicating whether the '@'-literal should be supported or not.
     *
     * @since 2.0b
     */
    private boolean supportAtLiteral = false;

    /**
     * <p>
     * Creates a new instance of <code>CStyleLineCounter</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given. '@'-literals are not supported when counting the lines.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     */
    public CStyleLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <p>
     * Creates a new instance of <code>CStyleLineCounter</code> class. The alias of this file processor and the file
     * matcher instance used by this file processor are given. A flag indicating whether '@'-literals are supported
     * or not is given as well.
     * </p>
     *
     * @param alias the unique alias for the processor.
     * @param matcher the file matcher instance used to determine whether a file should be processed by this file
     *        processor.
     * @param supportAtLiteral <code>true</code> if '@'-literals are supported; <code>false</code> otherwise.
     *
     * @throws NullPointerException if <code>alias</code> or <code>matcher</code> is <code>null</code>.
     * @throws IllegalArgumentException if <code>alias</code> is empty string.
     * @since 2.0b
     */
    public CStyleLineCounter(String alias, FileMatcher matcher, boolean supportAtLiteral) {
        this(alias, matcher);

        this.supportAtLiteral = supportAtLiteral;
    }

    /**
     * <p>
     * Creates a new instance of <code>CStyleLineCounter</code> class. The alias and the file matcher are loaded from
     * configuration. The flag indicating whether '@'-literals should be supported or not is loaded as well.
     * </p>
     *
     * @param property the property where configurations are read from.
     *
     * @throws NullPointerException if <code>property</code> is <code>null</code>.
     * @throws ConfigurationException if mandatory property is missing; or the file matcher instance cannot be
     *         created; or the alias is empty string.
     */
    public CStyleLineCounter(Property property) throws ConfigurationException {
        super(property);

        String value = property.getValue(SUPPORT_AT_LITERAL_PROPERTY_KEY);

        // If the value is "on", "yes" or "true", '@'-literals should be supported.
        supportAtLiteral = PROPERTY_VALUE_ON.equalsIgnoreCase(value) || PROPERTY_VALUE_TRUE.equalsIgnoreCase(value)
            || PROPERTY_VALUE_YES.equalsIgnoreCase(value);
    }

    /**
     * <p>
     * Gets the list of supported metrics which statistical data are collected against. Each time, a new list should
     * be returned. For this class, the returned list contains only one <code>Metric</code> instance with the name
     * 'Line Count'.
     * </p>
     *
     * @return the list of <code>Metric</code> instances representing supported metrics of this processor.
     */
    public List getSupportedMetrics() {
        List metrics = new ArrayList();

        metrics.add(SUPPORTED_METRIC);

        return metrics;
    }

    /**
     * <p>
     * Processes the given file. The number of C-style source code lines in the given file is counted. Whenever ';'
     * or '{' is encountered outside the string literals and comments, the following contexts are considered in a
     * new line. A simple state machine is used to process the content of the file.
     * </p>
     *
     * @param file the file where statistical data are collected from.
     *
     * @return the statistical result collected from the given file containing the number of C-style source code
     *         lines in the file.
     *
     * @throws NullPointerException if <code>file</code> is <code>null</code>.
     * @throws IOException if I/O error occurs.
     */
    public StatsResult process(File file) throws IOException {
        if (file == null) {
            throw new NullPointerException("file cannot be null.");
        }

        SimpleStateMachine ssm = new SimpleStateMachine();
        BufferedReader input = null;

        try {
            // Create an buffered text reader using the default encoding
            input = new BufferedReader(new FileReader(file));

            String line;

            while ((line = input.readLine()) != null) {
                // If a line can be read, process the line.
                ssm.process(line);
            }
        } finally {
            if (input != null) {
                input.close();
            }
        }

        // Create the result
        StatsResult result = new StatsResult();

        // Set the line count
        result.setStat(SUPPORTED_METRIC, ssm.getLines());

        return result;
    }

    /**
     * <p>
     * Gets a list of custom argument options which can be processed by this processor. Only one optional argument
     * can be processed by this processor, which is 'support_at_literal'. When the switch is specified, '@'-literals
     * are supported when counting the line number. The value of the switch is ignored.
     * </p>
     *
     * @return a list of <code>ArgumentOption</code> instances representing the switches can be processed by this
     *         processor.
     * @since 2.0b
     */
    public List proposeOptions() {
        List options = new ArrayList();

        options.add(SUPPORT_AT_LITERAL_OPTION);

        return options;
    }

    /**
     * <p>
     * Handles the argument received from the command line. When the switch 'support_at_literal' is
     * specified, '@'-literals are supported when counting line number. The value of the switch is ignored.
     * </p>
     *
     * @param arguments the argument names and values received from the command line.
     * @throws NullPointerException if <code>arguments</code> is <code>null</code>.
     * @throws IllegalArgumentExcpetion if the value of switch 'support_at_literal' is not <code>null</code> and
     *         not a string.
     * @since 2.0b
     */
    public void handleArguments(Map arguments) {
        supportAtLiteral = LineCounterHelper.isSwitchSpecified(arguments, SUPPORT_AT_LITERAL_PROPERTY_KEY);
    }

    /**
     * <p>
     * Defines a simple state machine used to count the number of C-style source code lines in a file. This state
     * machine supports single quoted literals, double quoted literals, escaped character in quoted literals, block
     * comments and line comments. Each time a line (CR/LF line) in the file is processed. After processing all
     * lines in the file, the number of C-style source code lines can be retrieved. '@'-quoted literals in C# can be
     * supported.
     * </p>
     *
     * <p>
     * This class is not thread-safe. However, at any time, only one thread can access one instance of this class.
     * </p>
     *
     * @author WishingBone, TCSDEVELOPER
     * @version 2.0b
     * @since 1.0
     */
    private class SimpleStateMachine {
        /** Represents the state when the next input character is not in comment or string literal. */
        private static final int STATE_START = 1;

        /**
         * Represents the state when the next input character is after a single slash '/'. If the next character is
         * '' or '/', it starts a block comment or line comment.
         */
        private static final int STATE_SLASH = 2;

        /**
         * Represents the state when the next input character is after an asterisk '' in a block comment. If the next
         * character is '/', it ends the block comment.
         */
        private static final int STATE_STAR = 3;

        /** Represents the state when the next input character is escaped in a single quoted literal. */
        private static final int STATE_SQ_BACKSLASH = 4;

        /** Represents the state when the next input character is escaped in a double quoted literal. */
        private static final int STATE_DQ_BACKSLASH = 5;

        /** Represents the state when the next input character is in a single quoted literal. */
        private static final int STATE_SINGLE_QUOTE = 6;

        /** Represents the state when the next input character is in a double quoted literal. */
        private static final int STATE_DOUBLE_QUOTE = 7;

        /** Represents the state when the next input character is in a block comment. */
        private static final int STATE_BLOCK_COMMENT = 8;

        /** Represents the state when the next input character is in a line comment. */
        private static final int STATE_LINE_COMMENT = 9;

        /**
         * Represents the state when the next input character is '@' outside a literal.
         *
         * @since 2.0b
         */
        private static final int STATE_AT = 10;

        /**
         * Represents the state when the next input character is in a '@'-quoted literal.
         *
         * @since 2.0b
         */
        private static final int STATE_AT_LITERAL = 11;

        /**
         * Represents the state when the next input character is a double quote in a '@'-quoted literal.
         *
         * @since 2.0b
         */
        private static final int STATE_DQ_AT_LITERAL = 12;

        /** Represents the line feed character. */
        private static final char LF = '\n';

        /**
         * Represents the current state of this state machine. At the beginning, the next character is always not in
         * a comment or literal.
         */
        private int state = STATE_START;

        /** Represents the number of C-style source code lines being processed. */
        private long lines = 0;

        /**
         * <p>
         * Creates a new instance of <code>SimpleStateMachine</code> class.
         * </p>
         */
        public SimpleStateMachine() {
        }

        /**
         * <p>
         * Processes a line in the file. Each character is fed into the state machine. After the line is processed, a
         * line feed character is fed into the state machine. Whenever the state machine enters the state of line
         * comment, the remaining characters in the line is discarded, and the line fed character is fed.
         * </p>
         *
         * @param line the line in the file to be processed.
         *
         * @throws NullPointerException if <code>line</code> is <code>null</code>.
         */
        public void process(String line) {
            if (line == null) {
                throw new NullPointerException("line cannot be null.");
            }

            // Process each character in the line
            for (int i = 0; i < line.length(); ++i) {
                process(line.charAt(i));

                if (state == STATE_LINE_COMMENT) {
                    // If it is a line comment, the remaining characters of this line is discarded.
                    break;
                }
            }

            // Process a line feed after the line.
            process(LF);
        }

        /**
         * <p>
         * Processes a character as the input of the state machine. When the character '{' or ';' is not in comments
         * or literals, the number of C-style source code lines increases. The state transition can be seen in the
         * component specification.
         * </p>
         *
         * @param ch the next character to be processed.
         */
        private void process(char ch) {
            switch (state) {
            case STATE_START:

                switch (ch) {
                case '/':
                    state = STATE_SLASH;

                    break;

                case '\'':
                    state = STATE_SINGLE_QUOTE;

                    break;

                case '\"':
                    state = STATE_DOUBLE_QUOTE;

                    break;

                case ';':
                case '{':
                    ++lines;

                    break;

                case '@':

                    if (supportAtLiteral) {
                        state = STATE_AT;
                    }

                    break;
                }

                break;

            case STATE_SLASH:

                switch (ch) {
                case '*':
                    state = STATE_BLOCK_COMMENT;

                    break;

                case '/':
                    state = STATE_LINE_COMMENT;

                    break;

                default:
                    state = STATE_START;

                    break;
                }

                break;

            case STATE_STAR:

                switch (ch) {
                case '/':
                    state = STATE_START;

                    break;

                case '*':
                    state = STATE_STAR;

                    break;

                default:
                    state = STATE_BLOCK_COMMENT;

                    break;
                }

                break;

            case STATE_SQ_BACKSLASH:
                state = STATE_SINGLE_QUOTE;

                break;

            case STATE_DQ_BACKSLASH:
                state = STATE_DOUBLE_QUOTE;

                break;

            case STATE_SINGLE_QUOTE:

                switch (ch) {
                case '\\':
                    state = STATE_SQ_BACKSLASH;

                    break;

                case '\'':
                    state = STATE_START;

                    break;
                }

                break;

            case STATE_DOUBLE_QUOTE:

                switch (ch) {
                case '\\':
                    state = STATE_DQ_BACKSLASH;

                    break;

                case '"':
                    state = STATE_START;

                    break;
                }

                break;

            case STATE_LINE_COMMENT:

                if (ch == LF) {
                    state = STATE_START;
                }

                break;

            case STATE_BLOCK_COMMENT:

                if (ch == '*') {
                    state = STATE_STAR;
                }

                break;

            case STATE_AT:

                switch (ch) {
                case '/':
                    state = STATE_SLASH;

                    break;

                case '\'':
                    state = STATE_SINGLE_QUOTE;

                    break;

                case '\"':
                    state = STATE_AT_LITERAL;

                    break;

                case ';':
                case '{':
                    ++lines;

                    break;

                case '@':
                    break;

                default:
                    state = STATE_START;

                    break;
                }
                break;

            case STATE_AT_LITERAL:

                if (ch == '"') {
                    state = STATE_DQ_AT_LITERAL;
                }

                break;

            case STATE_DQ_AT_LITERAL:

                switch (ch) {
                case '/':
                    state = STATE_SLASH;

                    break;

                case '\'':
                    state = STATE_SINGLE_QUOTE;

                    break;

                case '\"':
                    state = STATE_AT_LITERAL;

                    break;

                case ';':
                case '{':
                    ++lines;

                    break;

                case '@':

                    if (supportAtLiteral) {
                        state = STATE_AT;
                    }

                    break;

                default:
                    state = STATE_START;

                    break;
                }

                break;
            }
        }

        /**
         * <p>
         * Gets the number of C-style source code lines being processed.
         * </p>
         *
         * @return the number of C-style source code lines being processed.
         */
        public long getLines() {
            return lines;
        }
    }
}

