/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.util.datavalidator.ObjectValidator;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * <p>
 * Tests functionality and error cases of <code>ArgumentOption</code> class. Since this class is immutable, the accuracy
 * tests of property getters are combined into the accuracy test of the constructor.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 2.0b
 */
public class ArgumentOptionTestCase extends TestCase {
    /** Represents a valid name of argument option. */
    private static final String NAME = "name";

    /** Represents a valid help message of argument option. */
    private static final String HELP_MESSAGE = "help message";

    /** Represents a valid <code>ObjectValidator</code> instance. */
    private static final ObjectValidator VALIDATOR = new MockObjectValidator();

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean)</code> when the given name is <code>null</code>.
     * Expect NullPointerException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanNameNull() {
        try {
            new ArgumentOption(null, HELP_MESSAGE, true);
            fail("The name is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean)</code> when the given name is empty string.
     * Expect IllegalArgumentException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanNameEmpty() {
        try {
            new ArgumentOption("  ", HELP_MESSAGE, true);
            fail("The name is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean)</code> when the given help message is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanMessageNull() {
        try {
            new ArgumentOption(NAME, null, true);
            fail("The help message is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean)</code> when the given help messageis empty
     * string. Expect IllegalArgumentException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanMessageEmpty() {
        try {
            new ArgumentOption(NAME, "  ", true);
            fail("The help message is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code> when the given name is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValiadtorNameNull() {
        try {
            new ArgumentOption(null, HELP_MESSAGE, true, VALIDATOR);
            fail("The name is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code> when the given name is
     * empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValiadtorNameEmpty() {
        try {
            new ArgumentOption("  ", HELP_MESSAGE, true, VALIDATOR);
            fail("The name is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code> when the given help
     * message is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValidatorMessageNull() {
        try {
            new ArgumentOption(NAME, null, true, VALIDATOR);
            fail("The help message is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code> when the given help
     * message is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValidatorMessageEmpty() {
        try {
            new ArgumentOption(NAME, "  ", true, VALIDATOR);
            fail("The help message is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code> when the given validator
     * is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValidatorValidatorNull() {
        try {
            new ArgumentOption(NAME, HELP_MESSAGE, true, null);
            fail("The validator is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>ArgumentOption(String, String, boolean)</code>. The name, help message and
     * required flag should be set as given. The validator should be <code>null</code>.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanAccuracy() {
        ArgumentOption option = new ArgumentOption(NAME, HELP_MESSAGE, true);

        // Verify properties
        assertEquals("The name should be 'name'.", NAME, option.getSwitchName());
        assertEquals("The help message should be 'help message'.", HELP_MESSAGE, option.getHelpMessage());
        assertTrue("The required flag should be true.", option.isRequired());
        assertNull("The validator should be null.", option.getValidator());
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>ArgumentOption(String, String, boolean, ObjectValidator)</code>. The name,
     * help message, required flag and validator should be set as given.
     * </p>
     */
    public void testArgumentOptionStringStringBooleanValidatorAccuracy() {
        ArgumentOption option = new ArgumentOption(NAME, HELP_MESSAGE, false, VALIDATOR);

        // Verify properties
        assertEquals("The name should be 'name'.", NAME, option.getSwitchName());
        assertEquals("The help message should be 'help message'.", HELP_MESSAGE, option.getHelpMessage());
        assertFalse("The required flag should be false.", option.isRequired());
        assertEquals("The validator should be the same as given.", VALIDATOR, option.getValidator());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(ArgumentOptionTestCase.class);
    }
}




