/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.MockFileNameMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.PrintStream;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;


/**
 * <p>
 * Tests functionality and error cases of <code>ExtendedFileStatistics</code> class. The constructors of this class
 * soly delegate the super class' constructors. Therefore, they are not tested here.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class ExtendedFileStatisticsTestCase extends TestCase {
    /** Represents the <code>ExtendedFileStatistics</code> instance used in tests. */
    private ExtendedFileStatistics fileStat;

    /** Represents the list of files to be deleted. */
    private List filesToDelete;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The <code>ExtendedFileStatistics</code> instance
     * uses <code>BasicStatsReporting</code> instance. It contains one text line counter counting lines for XML
     * files. The list of files to be deleted is created.
     * </p>
     */
    protected void setUp() {
        fileStat = new ExtendedFileStatistics();

        fileStat.setReporting(new BasicStatsReporting());
        fileStat.addFileProcessor(new TextLineCounter("processor1", new ExtensionMatcher("xml")));

        filesToDelete = new ArrayList();
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The list of deleting files is deleted.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        fileStat = null;

        // Delete files
        Iterator iter = filesToDelete.iterator();

        while (iter.hasNext()) {
            ((File) iter.next()).delete();
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File)</code> when the given directory to be processed is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileNull() throws Exception {
        try {
            fileStat.processDirectory(null);
            fail("The directory to be processed is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File)</code> when the given directory to be processed does not
     * exist. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileNotExist() throws Exception {
        try {
            fileStat.processDirectory(new File("notexist"));
            fail("The directory to be processed does not exist, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File)</code> when the given directory to be processed is actually a
     * file. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileIsFile() throws Exception {
        try {
            fileStat.processDirectory(new File("test_files/error"));
            fail("The directory to be processed is a file, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File, boolean)</code> when the given directory to be processed is
     * <code>null</code>. Expect NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanNull() throws Exception {
        try {
            fileStat.processDirectory(null, false);
            fail("The directory to be processed is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File, boolean)</code> when the given directory to be processed does
     * not exist. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanNotExist()
        throws Exception {
        try {
            fileStat.processDirectory(new File("notexist"), false);
            fail("The directory to be processed does not exist, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File, boolean)</code> when the given directory to be processed is
     * actually a file. Expect IllegalArgumentException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanIsFile()
        throws Exception {
        try {
            fileStat.processDirectory(new File("test_files/error"), false);
            fail("The directory to be processed is a file, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>processDirectory(File)</code> when processing error occurs. Expect FileStatisticsException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileProcessingError()
        throws Exception {
        fileStat.addFileProcessor(new TextLineCounter("error", new MockFileNameMatcher()));

        try {
            fileStat.processDirectory(new File("test_files"));
            fail("Processing error occurs, should throw FileStatisticsException.");
        } catch (FileStatisticsException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>processDirectory(File, boolean)</code> when processing error occurs. Expect
     * FileStatisticsException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanProcessingError()
        throws Exception {
        fileStat.addFileProcessor(new TextLineCounter("error", new MockFileNameMatcher()));

        try {
            fileStat.processDirectory(new File("test_files"), false);
            fail("Processing error occurs, should throw FileStatisticsException.");
        } catch (FileStatisticsException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>printReport()</code> when error occurs generating the report. Expect StatsReportingException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testPrintReportError() throws Exception {
        fileStat.setReporting(new XmlStatsReporting(new File("test_files/TextReport.txt")));

        // Process a directory
        fileStat.processFile(new File("test_files"));

        try {
            fileStat.printReport();
            fail("Error occurs when generating the report, should throw StatsReportingException.");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>printReport(File)</code> when error occurs generating the report. Expect StatsReportingException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testPrintReportFileError() throws Exception {
        fileStat.setReporting(new XmlStatsReporting(new File("test_files/TextReport.txt")));

        // Process a directory
        fileStat.processFile(new File("test_files"));

        File file = new File("test_files/report.tmp");

        filesToDelete.add(file);

        try {
            fileStat.printReport(file);
            fail("Error occurs when generating the report, should throw StatsReportingException.");
        } catch (StatsReportingException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests <code>printReport(File)</code> when I/O error occurs printing the report. Expect IOException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testPrintReportFileIOError() throws Exception {
        fileStat.setReporting(new XmlStatsReporting(new File("test_files/TextReport.txt")));

        // Process a directory
        fileStat.processFile(new File("test_files"));

        File file = new File("test_files/notexist/report.tmp");

        filesToDelete.add(file);

        try {
            fileStat.printReport(file);
            fail("I/O error occurs printing the report, should throw IOException.");
        } catch (IOException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File)</code>. Only files in the directory should be processed, not
     * in its sub-directories. The generated report is used to verify this.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileAccuracy() throws Exception {
        fileStat.processDirectory(new File("test_files"));

        // Generate a report and verify
        String report = fileStat.getReporting().generateReport();

        TestHelper.assertEquals("Only files in the directory should be processed.",
            new File("test_files/DirectoryOnly.txt"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File, boolean)</code> when the processing is non-recursive. Only
     * files in the directory should be processed, not in its sub-directories. The generated report is used to
     * verify this.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanNonRecursiveAccuracy()
        throws Exception {
        fileStat.processDirectory(new File("test_files"), false);

        // Generate a report and verify
        String report = fileStat.getReporting().generateReport();

        TestHelper.assertEquals("Only files in the directory should be processed.",
            new File("test_files/DirectoryOnly.txt"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>processDirectory(File, boolean)</code> when the processing is recursive. All files in
     * the directory as well as in all sub-directories should be processed. The generated report is used to verify
     * this.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testProcessDirectoryFileBooleanRecursiveAccuracy()
        throws Exception {
        fileStat.processDirectory(new File("test_files"), true);

        // Generate a report and verify
        String report = fileStat.getReporting().generateReport();

        TestHelper.assertEquals("All files in the directory and its sub-directories should be processed.",
            new File("test_files/DirectoryAll.txt"), report);
    }

    /**
     * <p>
     * Tests accuracy of <code>printReport()</code>. The printed report should be the same as the generated report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testPrintReportAccuracy() throws Exception {
        fileStat.processDirectory(new File("test_files"), true);

        // Save the original output stream
        PrintStream out = System.out;
        ByteArrayOutputStream content = new ByteArrayOutputStream();
        PrintStream save = new PrintStream(content);

        try {
            System.setOut(save);

            // Print the report
            fileStat.printReport();
        } finally {
            System.setOut(out);
        }

        String report = new String(content.toByteArray());

        // Verify the printed report
        assertEquals("The printed report should be the same as generated.", report,
            fileStat.getReporting().generateReport());
    }

    /**
     * <p>
     * Tests accuracy of <code>printReport(File)</code>. The printed report should be the same as the generated
     * report.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testPrintReportFileAccuracy() throws Exception {
        fileStat.processDirectory(new File("test_files"), true);

        File file = new File("test_files/report.tmp");

        filesToDelete.add(file);

        // Print the report
        fileStat.printReport(file);

        // Verify the printed report
        TestHelper.assertEquals("The printed report should be the same as generated.", file,
            fileStat.getReporting().generateReport());
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(ExtendedFileStatisticsTestCase.class);
    }
}
