/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * <p>
 * Tests functionality and error cases of <code>Metric</code> class. Since this class is immutable, the accuracy
 * tests of property getters are combined into the accuracy test of the constructor.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class MetricTestCase extends TestCase {
    /** Represents a valid name of metric. */
    private static final String NAME = "name";

    /** Represents a valid display name of metric. */
    private static final String DISPLAY_NAME = "display name";

    /** Represents a valid description of metric. */
    private static final String DESCRIPTION = "description";

    /** Represents the <code>Metric</code> instance used in tests. */
    private Metric metric;

    /**
     * <p>
     * Sets up the test environment. The test instance is created.
     * </p>
     */
    protected void setUp() {
        metric = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed.
     * </p>
     */
    protected void tearDown() {
        metric = null;
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given name is <code>null</code>. Expect NullPointerException.
     * </p>
     */
    public void testMetricNameNull() {
        try {
            new Metric(null, DISPLAY_NAME, DESCRIPTION);
            fail("The name is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given name is empty string. Expect IllegalArgumentException.
     * </p>
     */
    public void testMetricNameEmpty() {
        try {
            new Metric("  ", DISPLAY_NAME, DESCRIPTION);
            fail("The name is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given display name is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testMetricDisplayNameNull() {
        try {
            new Metric(NAME, null, DESCRIPTION);
            fail("The display name is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given display name is empty string. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testMetricDisplayNameEmpty() {
        try {
            new Metric(NAME, "  ", DESCRIPTION);
            fail("The display name is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given description is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testMetricDescriptionNull() {
        try {
            new Metric(NAME, DISPLAY_NAME, null);
            fail("The description is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>Metric</code> when the given description is empty string. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testMetricDescriptionEmpty() {
        try {
            new Metric(NAME, DISPLAY_NAME, "  ");
            fail("The description is empty string, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>Metric</code>. The getters of name, display name and description should
     * retrieve the same value as provided in the constructor.
     * </p>
     */
    public void testMetricAccuracy() {
        metric = new Metric(NAME, DISPLAY_NAME, DESCRIPTION);

        // Verify properties
        assertEquals("The name should be the same.", NAME, metric.getName());
        assertEquals("The display name should be the same.", DISPLAY_NAME, metric.getDisplayName());
        assertEquals("The description should be the same.", DESCRIPTION, metric.getDescription());
    }

    /**
     * <p>
     * Tests accuracy of <code>hashCode</code>. The hash code of metric instance should be the same as the hash code
     * of the name of the metric.
     * </p>
     */
    public void testHashCodeAccuracy() {
        assertEquals("The hash code of metric instance should be the same as the hash code of the name.",
            NAME.hashCode(), metric.hashCode());
    }

    /**
     * <p>
     * Tests accuracy of <code>equals</code> when the given object is <code>null</code>. It should return
     * <code>false</code>, since <code>null</code> cannot equal to any non-null instance.
     * </p>
     */
    public void testEqualsNullAccuracy() {
        assertFalse("Null cannot equal to any metric instance.", metric.equals(null));
    }

    /**
     * <p>
     * Tests accuracy of <code>equals</code> when the given object is not an instance of <code>Metric</code> class.
     * It should return <code>false</code>, since instances of different classes cannot be the same.
     * </p>
     */
    public void testEqualsNotMetricAccuracy() {
        assertFalse("Instances of different classes cannot be the same.", metric.equals(new Object()));
    }

    /**
     * <p>
     * Tests accuracy of <code>equals</code> when the given object is an instance of <code>Metric</code> class, but
     * has a different name. It should return <code>false</code>.
     * </p>
     */
    public void testEqualsMetricDifferentNameAccuracy() {
        Metric metric2 = new Metric("different name", DISPLAY_NAME, DESCRIPTION);

        assertFalse("Metrics with different names cannot be the same.", metric.equals(metric2));
    }

    /**
     * <p>
     * Tests accuracy of <code>equals</code> when the given object is an instance of <code>Metric</code> class, and
     * has the same name. It should return <code>true</code>.
     * </p>
     */
    public void testEqualsMetricSameNameAccuracy() {
        Metric metric2 = new Metric(NAME, "different display name", "different description");

        assertTrue("Metrics with the same name should be the same.", metric.equals(metric2));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(MetricTestCase.class);
    }
}




