/**
 * Copyright (c) 2005, TopCoder, Inc. All rights reserved.
 */
package com.topcoder.file.statistics.accuracytests.matcher;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.accuracytests.TestHelper;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;


/**
 * <p>
 * Accuracy tests for ExtensionMatcher.
 * </p>
 *
 * @author cucu
 * @version 1.0
 */
public class ExtensionMatcherAccuracyTests extends TestCase {
    /**
     * Test the Match method against a single extension.
     * It test with many different filenames, covering corner cases.
     */
    public void testMatchWithStringConstructor() throws Exception  {
        FileMatcher matcher = new ExtensionMatcher("txt");

        assertTrue("file.txt should match with extension txt", matcher.match(new File("file.txt")));
        assertTrue("file.TxT should match with extension txt", matcher.match(new File("file.TxT")));
        assertTrue("file.TXT should match with extension txt", matcher.match(new File("file.TXT")));
        assertTrue(".txt should match with extension txt", matcher.match(new File(".txt")));
        assertTrue("c:/documents/version 1.0/hello.txt should match with extension txt", matcher.match(new File("c:/documents/version 1.0/hello.txt")));
        assertTrue("/documents/version 1.0/hello.world.txt should match with extension txt", matcher.match(new File("/documents/version 1.0/hello.world.txt")));

        assertFalse("txt should not match with extension txt", matcher.match(new File("txt")));
        assertFalse("file.txt_ should not match with extension txt", matcher.match(new File("file.txt_")));
        assertFalse("/my.txt/hello.bak should not match with extension txt", matcher.match(new File("/my.txt/hello.bak")));
        assertFalse("hello.txt.bak should not match with extension txt", matcher.match(new File("hello.txt.bak")));
    }


    /**
     * Test that the list constructor makes a copy of the list
     */
    public void testListConstructor() throws Exception {
        List extensions = new ArrayList();
        extensions.add("txt");

        FileMatcher matcher = new ExtensionMatcher(extensions);

        // this shouldn't affect the matcher
        extensions.add("bak");

        assertFalse("constructor is not making a copy", matcher.match(new File("file.bak")));
    }

    /**
     * Test the Match method against many extensions, using the constructor that takes a list of extensions.
     */
    public void testMatchWithListConstructor() throws Exception  {
        List extensions = new ArrayList();
        extensions.add("cPp");
        extensions.add("cs");
        extensions.add("JAVA");
        extensions.add("pas");

        FileMatcher matcher = new ExtensionMatcher(extensions);

        checkExtensions(matcher);
    }

    /**
     *  Test the Match method against many extensions, using the constructor that reads from the configuration file.
     *
     * @throws Exception to JUnit
     */
    public void testMatchWithConfigManagerConstructor() throws Exception  {
        TestHelper.loadConfig(TestHelper.MATCHERS_CONFIG);

        try {
            Property prop = ConfigManager.getInstance().getPropertyObject(TestHelper.MATCHER_NAMESPACE, "extension");
            FileMatcher matcher = new ExtensionMatcher(prop);
            checkExtensions(matcher);
        } finally {
            TestHelper.unloadNamespace(TestHelper.MATCHER_NAMESPACE);
        }
    }

    /**
     * Check that the matcher passed is matching well with the extensions: cpp, cs, java and pas.
     *
     * @param matcher the matcher to check
     */
    private void checkExtensions(FileMatcher matcher) throws Exception {
        assertTrue("HelloWorld.cpp should match with the extensions", matcher.match(new File("HelloWorld.cpp")));
        assertTrue("HelloWorld.cs should match with the extensions", matcher.match(new File("HelloWorld.cs")));
        assertTrue("HelloWorld.cS should match with the extensions", matcher.match(new File("HelloWorld.cS")));
        assertTrue("HelloWorld.java should match with the extensions", matcher.match(new File("HelloWorld.java")));
        assertTrue("HelloWorld.pas should match with the extensions", matcher.match(new File("HelloWorld.pas")));
        assertTrue("c:/HelloWorld.pas should match with the extensions", matcher.match(new File("c:/HelloWorld.pas")));
        assertTrue("/doc.java/HelloWorld.pas should match with the extensions", matcher.match(new File("/doc.java/HelloWorld.pas")));

        assertFalse("HelloWorld.~cpp should NOT match with the extensions", matcher.match(new File("HelloWorld.~cpp")));
        assertFalse("java should NOT match with the extensions", matcher.match(new File("java")));
        assertFalse("java.txt should NOT match with the extensions", matcher.match(new File("java.txt")));
        assertFalse("Hello.cpp.bak should NOT match with the extensions", matcher.match(new File("Hello.cpp.bak")));
        assertFalse("c:/program.xpas should NOT match with the extensions", matcher.match(new File("c:/program.xpas")));

    }

}
