/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.file.statistics.matcher;

import com.topcoder.file.statistics.ConfigurationException;

import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.Property;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.lang.reflect.Field;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;


/**
 * <p>
 * Tests functionality and error cases of <code>RegexMatcher</code> class. The <code>match(File)</code> method is
 * defined in its super class <code>FileNameMatcher</code>. The accuracy and error cases of this method is already
 * tested in the test case of that class. Therefore, here only the protected <code>match</code> method is tested.
 * </p>
 *
 * @author TCSDEVELOPER
 * @version 2.0b
 * @since 1.0
 */
public class RegexMatcherTestCase extends TestCase {
    /** Represents the namespace used in tests. */
    private static final String NAMESPACE = "test";

    /** Represents the property name used in tests. */
    private static final String PROPERTY = "test_property";

    /** Represents the <code>RegexMatcher</code> instance used in tests. */
    private RegexMatcher matcher;

    /** Represents the configuration manager instance used in tests. */
    private ConfigManager configManager;

    /**
     * <p>
     * Sets up the test environment. The test instance is created. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void setUp() throws Exception {
        configManager = ConfigManager.getInstance();

        // The matcher instance contains three patterns
        List list = new ArrayList();

        list.add(Pattern.compile("^a.*b$"));
        list.add(Pattern.compile("TxT"));
        list.add(Pattern.compile("\\.bak\\.cpp$"));
        matcher = new RegexMatcher(list);

        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }
    }

    /**
     * <p>
     * Cleans up the test environment. The test instance is disposed. The test namespace is removed.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    protected void tearDown() throws Exception {
        if (configManager.existsNamespace(NAMESPACE)) {
            configManager.removeNamespace(NAMESPACE);
        }

        matcher = null;
        configManager = null;
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Pattern)</code> when the given pattern is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPatternNull() throws Exception {
        try {
            new RegexMatcher((Pattern) null);
            fail("The pattern is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(List)</code> when the given pattern list is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     */
    public void testRegexMatcherListNull() {
        try {
            new RegexMatcher((List) null);
            fail("The pattern list is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(List)</code> when the given pattern list is empty. Expect
     * IllegalArgumentException.
     * </p>
     */
    public void testRegexMatcherListEmpty() {
        try {
            new RegexMatcher(new ArrayList());
            fail("The pattern list is empty, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(List)</code> when the given pattern list contains non-pattern instance.
     * Expect IllegalArgumentException.
     * </p>
     */
    public void testRegexMatcherListNotString() {
        List list = new ArrayList();

        list.add(new Object());

        try {
            new RegexMatcher(list);
            fail("The pattern list contains non-pattern instance, should throw IllegalArgumentException.");
        } catch (IllegalArgumentException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the given property is <code>null</code>. Expect
     * NullPointerException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyNull() throws Exception {
        try {
            new RegexMatcher((Property) null);
            fail("The property is null, should throw NullPointerException.");
        } catch (NullPointerException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the pattern list is missing in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyPatternMissing()
        throws Exception {
        configManager.add("RegexMatcher/Missing.xml");

        try {
            new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The pattern list is missing in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the pattern has wrong syntax in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyPatternInvalid()
        throws Exception {
        configManager.add("RegexMatcher/Error.xml");

        try {
            new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The pattern has wrong syntax in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests constructor <code>RegexMatcher(Property)</code> when the pattern is empty string in configuration.
     * Expect ConfigurationException.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyPatternEmpty()
        throws Exception {
        configManager.add("RegexMatcher/EmptyString.xml");

        try {
            new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));
            fail("The pattern is empty string in configuration, should throw ConfigurationException.");
        } catch (ConfigurationException e) {
            // good
        }
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>RegexMatcher(Pattern)</code>. The internal pattern list should contain
     * only one pattern, which is given as the argument.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPatternAccuracy() throws Exception {
        Pattern pattern = Pattern.compile("jAvA");

        matcher = new RegexMatcher(pattern);

        // Get the internal list
        Field field = RegexMatcher.class.getDeclaredField("patterns");
        List list;

        try {
            field.setAccessible(true);
            list = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify
        assertEquals("One pattern should exist.", 1, list.size());
        assertEquals("The pattern should be correct.", pattern, list.get(0));
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>RegexMatcher(List)</code>. The internal pattern list should contain all
     * patterns which are given as the argument. A copy of the given list should be made.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherListAccuracy() throws Exception {
        List list = new ArrayList();

        list.add(Pattern.compile("^a.*b$"));
        list.add(Pattern.compile("TxT"));
        list.add(Pattern.compile("\\.bak\\.cpp$"));

        matcher = new RegexMatcher(list);

        // Get the internal list
        Field field = RegexMatcher.class.getDeclaredField("patterns");
        List actual;

        try {
            field.setAccessible(true);
            actual = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // Verify copy
        assertNotSame("A copy of the list should be made.", list, actual);

        // Verify the list
        assertEquals("The pattern list should be correct.", list, actual);
    }

    /**
     * <p>
     * Tests accuracy of constructor <code>RegexMatcher(Property)</code>. The internal pattern list should contain
     * all patterns which are specified in the configuration.
     * </p>
     *
     * @throws Exception pass any unexpected exception to JUnit.
     */
    public void testRegexMatcherPropertyAccuracy() throws Exception {
        configManager.add("RegexMatcher/Valid.xml");

        matcher = new RegexMatcher(configManager.getPropertyObject(NAMESPACE, PROPERTY));

        // Get the internal list
        Field field = RegexMatcher.class.getDeclaredField("patterns");
        List list;

        try {
            field.setAccessible(true);
            list = (List) field.get(matcher);
        } finally {
            field.setAccessible(false);
        }

        // The expected regular expression list.
        List expected = new ArrayList();

        expected.add("^a.*b$");
        expected.add("TxT");
        expected.add("\\.bak\\.cpp$");

        // Build the actual regular expression list.
        List actual = new ArrayList();
        Iterator iter = list.iterator();

        while (iter.hasNext()) {
            actual.add(((Pattern) iter.next()).pattern());
        }

        // Verify the list
        assertEquals("The pattern list should be correct.", expected, actual);
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given file name matches at least one pattern in the pattern
     * list.
     * </p>
     */
    public void testMatchStringExistAccuracy() {
        assertTrue("The file name matches at least one pattern in the pattern list.",
            matcher.match("whatsoever.bak.cpp"));
    }

    /**
     * <p>
     * Tests accuracy of <code>match</code> when the given file name does not match any pattern in the pattern list.
     * </p>
     */
    public void testMatchStringNotExistAccuracy() {
        assertFalse("The file name does not match any pattern in the pattern list.", matcher.match("whatsoever.bak"));
    }

    /**
     * <p>
     * Aggragates all tests in this class.
     * </p>
     *
     * @return test suite aggragating all tests.
     */
    public static Test suite() {
        return new TestSuite(RegexMatcherTestCase.class);
    }
}




